// ██╗███╗   ███╗██████╗  ██████╗ ██████╗ ████████╗███████╗
// ██║████╗ ████║██╔══██╗██╔═══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║██╔████╔██║██████╔╝██║   ██║██████╔╝   ██║   ███████╗
// ██║██║╚██╔╝██║██╔═══╝ ██║   ██║██╔══██╗   ██║   ╚════██║
// ██║██║ ╚═╝ ██║██║     ╚██████╔╝██║  ██║   ██║   ███████║
// ╚═╝╚═╝     ╚═╝╚═╝      ╚═════╝ ╚═╝  ╚═╝   ╚═╝   ╚══════╝
import { ActionFormData, ModalFormData, FormCancelationReason } from "@minecraft/server-ui";
import { Database } from "index.js";
import { isitem, isblock } from "texturePath.js";
import { world, system } from "@minecraft/server";


//  ██████╗ ██████╗ ███╗   ██╗███████╗████████╗███████╗
// ██╔════╝██╔═══██╗████╗  ██║██╔════╝╚══██╔══╝██╔════╝
// ██║     ██║   ██║██╔██╗ ██║███████╗   ██║   ███████╗
// ██║     ██║   ██║██║╚██╗██║╚════██║   ██║   ╚════██║
// ╚██████╗╚██████╔╝██║ ╚████║███████║   ██║   ███████║
//  ╚═════╝ ╚═════╝ ╚═╝  ╚═══╝╚══════╝   ╚═╝   ╚══════╝
const CrateDisplayName1 = "§aC§br§aa§bt§ae";
const CrateDisplayName2 = "§bC§ar§ba§at§be";
const LegendaryCrateDisplayName1 = "§6L§re§6g§re§6n§rd§6a§rr§6y \nC§6r§ra§6t§re";
const LegendaryCrateDisplayName2 = "§rL§6e§rg§6e§rn§6d§ra§6r§ry \n§6C§rr§6a§rt§6e";

const GalaxyCrateDisplayName1 = "§dG§ra§dl§ra§dx§ry \n§rC§dr§ra§dt§re";
const GalaxyCrateDisplayName2 = "§rG§da§rl§da§rx§dy \n§dC§rr§da§rt§de";;


const PowerCrateDisplayName1 = "§3P§ro§3w§re§3r \nC§3r§ra§3t§re";
const PowerCrateDisplayName2 = "§rP§3o§rw§3e§rr§3 \n§3C§rr§3a§rt§3e";

const CrazyCrateDisplayName1 = "§9C§rr§9a§rz§9y \nC§9r§ra§9t§re";
const CrazyCrateDisplayName2 = "§rC§9r§ra§9z§ry \n§9C§rr§9a§rt§9e";


const MysticCrateDisplayName1 = "§4M§ry§4s§rt§4i§rc \nC§4r§ra§4t§re";
const MysticCrateDisplayName2 = "§rM§4y§rs§4t§ri§4c \n§4C§rr§4a§rt§4e";
let frameToggle = 0;

system.runInterval(() => {
    const dimension = world.getDimension('overworld');
    frameToggle = (frameToggle + 1) % 2; // Flip between 0 and 1
    const crateTypes = [
        {
            type: 'kittys:chest',
            displayNames: [CrateDisplayName1, CrateDisplayName2]
        },
        {
            type: 'kittys:legendarychest',
            displayNames: [LegendaryCrateDisplayName1, LegendaryCrateDisplayName2]
        },
        {
            type: 'kittys:galaxychest',
            displayNames: [GalaxyCrateDisplayName1, GalaxyCrateDisplayName2]
        },
        {
            type: 'kittys:powerchest',
            displayNames: [PowerCrateDisplayName1, PowerCrateDisplayName2]
        },
        {
            type: 'kittys:crazychest',
            displayNames: [CrazyCrateDisplayName1, CrazyCrateDisplayName2]
        },
        {
            type: 'kittys:mysticchest',
            displayNames: [MysticCrateDisplayName1, MysticCrateDisplayName2]
        }
    ];
    for (const crate of crateTypes) {
        const chests = dimension.getEntities({ type: crate.type });
        for (const chest of chests) {
            if (!chest.isValid()) continue;
            chest.nameTag = crate.displayNames[frameToggle];
        }
    }
}, 20);
async function forceShow(player, form, timeout = Infinity) {
    const startTick = system.currentTick;

    while ((system.currentTick - startTick) < timeout) {
        const result = await form.show(player);
        if (result.cancelationReason !== FormCancelationReason.UserBusy) return result;
    }

    throw new Error(`Timed out after ${timeout} ticks`);
}
const SettingsDb = new Database("Settings");
const ItemsDb = new Database("Items");
const cooldowns = new Map();

const LegendarySettingsDb = new Database("LegendarySettings");
const LegendaryItemsDb = new Database("LegendaryItems");
const Legendarycooldowns = new Map();

const GalaxySettingsDb = new Database("GalaxySettings");
const GalaxyItemsDb = new Database("GalaxyItems");
const Galaxycooldowns = new Map();

const CrazySettingsDb = new Database("CrazySettings");
const CrazyItemsDb = new Database("CrazyItems");
const Crazycooldowns = new Map();

const PowerSettingsDb = new Database("PowerSettings");
const PowerItemsDb = new Database("PowerItems");
const Powercooldowns = new Map();

const MysticSettingsDb = new Database("MysticSettings");
const MysticItemsDb = new Database("MysticItems");
const Mysticcooldowns = new Map();

const prizes = [
    { item: 'netherite_sword', quantity: 1, weight: 5 },   // Powerful weapon
    { item: 'diamond_axe', quantity: 1, weight: 3 },       // Strong melee weapon
    { item: 'bow', quantity: 1, weight: 3 },               // Ranged weapon
    { item: 'trident', quantity: 1, weight: 2 },           // Versatile weapon
    { item: 'crossbow', quantity: 1, weight: 2 },          // Ranged weapon
    { item: 'shield', quantity: 1, weight: 4 },            // Defensive item
    { item: 'arrow', quantity: 16, weight: 3 },            // Ammunition for ranged weapons
    { item: 'firework_rocket', quantity: 6, weight: 1 },   // Explosive ammo for crossbow
    { item: 'golden_apple', quantity: 2, weight: 3 },      // Combat healing
    { item: 'tipped_arrow', quantity: 12, weight: 1 }      // Special arrows
];
const Legendaryprizes = [
    { item: 'elytra', quantity: 1, weight: 2 },            // Rare flying item
    { item: 'dragon_egg', quantity: 1, weight: 0.5 },      // Unique item
    { item: 'netherite_block', quantity: 1, weight: 1 },   // High-value block
    { item: 'beacon', quantity: 1, weight: 2 },            // Rare structure block
    { item: 'diamond', quantity: 16, weight: 3 },          // High-value resource
    { item: 'nether_star', quantity: 1, weight: 1 },       // Crafting material for beacons
    { item: 'record_13', quantity: 1, weight: 4 },         // Legendary music disc
    { item: 'trident', quantity: 1, weight: 2 },            // Rare survival item
    { item: 'shulker_box', quantity: 1, weight: 3 },       // Portable storage
    { item: 'enchanting_table', quantity: 1, weight: 2 }   // Useful for enchanting items
];
const Galaxyprizes = [
    { item: 'elytra', quantity: 1, weight: 3 },            // Flying item for space travel
    { item: 'firework_rocket', quantity: 10, weight: 2 },  // Rockets for flight boost
    { item: 'nether_star', quantity: 1, weight: 1 },       // Star-themed item
    { item: 'phantom_membrane', quantity: 5, weight: 1 },  // Used to repair Elytra
    { item: 'obsidian', quantity: 16, weight: 3 },         // Space-like dark material
    { item: 'trident', quantity: 2, weight: 0.5 },          // Portal to the End dimension
    { item: 'shulker_shell', quantity: 4, weight: 2 },     // Space creature material
    { item: 'ender_pearl', quantity: 16, weight: 2 },      // Ender transportation item
    { item: 'chorus_fruit', quantity: 8, weight: 2 },      // Teleportation effect
    { item: 'glowstone', quantity: 12, weight: 3 }         // Glowing block from another dimension
];
const Crazyprizes = [
    { item: 'tnt', quantity: 8, weight: 3 },               // Explosive fun
    { item: 'fire_charge', quantity: 6, weight: 2 },       // Fire-starting item
    { item: 'cake', quantity: 1, weight: 4 },              // Random and fun item
    { item: 'slime_block', quantity: 8, weight: 2 },       // Bouncy and fun
    { item: 'honey_block', quantity: 8, weight: 2 },       // Sticky, crazy block
    { item: 'fishing_rod', quantity: 1, weight: 2 },       // Random activity item
    { item: 'firework_rocket', quantity: 16, weight: 2 },  // Fireworks for excitement
    { item: 'lead', quantity: 4, weight: 2 },              // For capturing animals
    { item: 'snowball', quantity: 16, weight: 3 },         // Fun throwing item
    { item: 'saddle', quantity: 1, weight: 3 }             // For riding random mobs
];
const Powerprizes = [
    { item: 'netherite_chestplate', quantity: 1, weight: 4 }, // Ultimate defense
    { item: 'potion', quantity: 3, weight: 3 },               // Strength potion
    { item: 'diamond_sword', quantity: 1, weight: 3 },        // Powerful weapon
    { item: 'anvil', quantity: 1, weight: 2 },                // Useful for repairs
    { item: 'iron_block', quantity: 16, weight: 2 },          // Crafting material for strength
    { item: 'golden_apple', quantity: 4, weight: 2 },         // Healing item
    { item: 'enchanted_golden_apple', quantity: 1, weight: 1 },// Rare healing item
    { item: 'nether_star', quantity: 1, weight: 1 },          // Powerful crafting material
    { item: 'trident', quantity: 1, weight: 2 },              // Survival power
    { item: 'beacon', quantity: 1, weight: 1 }                // Symbol of power
];
const Mysticprizes = [
    { item: 'enchanting_table', quantity: 1, weight: 3 },     // Enchanting station
    { item: 'book', quantity: 4, weight: 4 },                 // Magic books
    { item: 'experience_bottle', quantity: 16, weight: 2 },   // Experience for enchanting
    { item: 'trident', quantity: 1, weight: 2 },               // Magical survival item
    { item: 'lapis_block', quantity: 16, weight: 3 },         // Enchantment fuel
    { item: 'end_crystal', quantity: 1, weight: 1 },          // Mystical item for reviving Ender Dragon
    { item: 'potion', quantity: 4, weight: 2 },               // Potions of healing or magic
    { item: 'emerald_block', quantity: 4, weight: 2 },        // Valuable mystical currency
    { item: 'dragon_breath', quantity: 1, weight: 1 },        // Magical crafting material
    { item: 'glowstone_dust', quantity: 16, weight: 2 }       // Mystical light source
];
function setCooldown(player) {
    cooldowns.set(player, Date.now());
}
function LegendarysetCooldown(player) {
    Legendarycooldowns.set(player, Date.now());
}
function GalaxysetCooldown(player) {
    Galaxycooldowns.set(player, Date.now());
}
function CrazysetCooldown(player) {
    Crazycooldowns.set(player, Date.now());
}
function PowersetCooldown(player) {
    Powercooldowns.set(player, Date.now());
}
function MysticsetCooldown(player) {
    Mysticcooldowns.set(player, Date.now());
}
function playCrateSound(ToggleLegendarySoundFileValue, player) {
    switch (ToggleLegendarySoundFileValue) {
        case 1:
            player.playSound('kittys.crate.open', { volume: 1 });
            break;
        case 2:
            player.playSound('kittys.legendarycrate.open', { volume: 1 });
            break;
        case 3:
            player.playSound('kittys.galaxycrate.open', { volume: 1 });
            break;
        case 4:
            player.playSound('kittys.crazycrate.open', { volume: 1 });
            break;
        case 5:
            player.playSound('kittys.mysticcrate.open', { volume: 1 });
            break;
        case 6:
            player.playSound('kittys.powercrate.open', { volume: 1 });
            break;
        case 7:
            player.playSound('kittys.futurecrates1.open', { volume: 1 });
            break;
        case 8:
            player.playSound('kittys.futurecrates2.open', { volume: 1 });
            break;
        case 9:
            player.playSound('kittys.futurecrates3.open', { volume: 1 });
            break;
        case 10:
            player.playSound('kittys.futurecrates4.open', { volume: 1 });
            break;
        case 11:
            player.playSound('kittys.futurecrates5.open', { volume: 1 });
            break;
        case 12:
            player.playSound('kittys.futurecrates6.open', { volume: 1 });
            break;
        default:
            player.playSound('kittys.crate.open', { volume: 1 });
            break;
    }
}

world.beforeEvents.itemUse.subscribe(data => {
    const player = data.source;
    if (!player || data.itemStack.typeId !== "kittys:removal_tool") return;
    system.run(() => {
        (async () => {
            const overworld = world.getDimension('overworld');
            const CrateLeaderboards = overworld.getEntities({ type: 'kittys:hologram', tags: ['kittysCrateLeaderboard'] });
            const Crate = overworld.getEntities({ type: 'kittys:chest' });
            const legendarychest = overworld.getEntities({ type: 'kittys:legendarychest' });
            const galaxychest = overworld.getEntities({ type: 'kittys:galaxychest' });
            const powerchest = overworld.getEntities({ type: 'kittys:powerchest' });
            const crazychest = overworld.getEntities({ type: 'kittys:crazychest' });
            const mysticchest = overworld.getEntities({ type: 'kittys:mysticchest' });
            const allEntities = CrateLeaderboards.concat(Crate, legendarychest, galaxychest, powerchest, crazychest, mysticchest);

            if (allEntities.length === 0) {
                player.sendMessage('§cNo Crate Leaderboards or Crates To Remove!✨');
                player.playSound("note.bass", { volume: 0.5 });
                return;
            }

            let form = new ActionFormData()
                .title("Remove Crate Leaderboard or Crate")
                .body("Select a Crate Leaderboard or Crate to remove:");

            const getEntityLabel = (entity, label) => {
                return `§l§cRemove ${label}§r✨\n§7§o[ X:${entity.location.x} Y:${entity.location.y} Z:${entity.location.z} ]✨`;
            };

            CrateLeaderboards.forEach(entity => form.button(getEntityLabel(entity, "Crate Leaderboard"), "textures/ui/cancel"));
            Crate.forEach(entity => form.button(getEntityLabel(entity, "Crate"), "textures/ui/cancel"));
            legendarychest.forEach(entity => form.button(getEntityLabel(entity, "Legendary Crate"), "textures/ui/cancel"));
            galaxychest.forEach(entity => form.button(getEntityLabel(entity, "Galaxy Crate"), "textures/ui/cancel"));
            powerchest.forEach(entity => form.button(getEntityLabel(entity, "Power Crate"), "textures/ui/cancel"));
            crazychest.forEach(entity => form.button(getEntityLabel(entity, "Crazy Crate"), "textures/ui/cancel"));
            mysticchest.forEach(entity => form.button(getEntityLabel(entity, "Mystic Crate"), "textures/ui/cancel"));

            form.show(player).then(async (result) => {
                if (result.canceled) return;

                let selectedEntity = allEntities[result.selection];
                if (!selectedEntity) {
                    player.sendMessage('§cError: Invalid selection!✨');
                    return;
                }

                // Execute kill command to remove the selected entity
                await overworld.runCommandAsync(`execute at @e positioned ${selectedEntity.location.x} ${selectedEntity.location.y} ${selectedEntity.location.z} run kill @e[r=1,c=1]`);
                let label = selectedEntity.hasTag('kittysCrateLeaderboard') ? "Crate Leaderboard" : "Crate";
                player.sendMessage(`§cRemoved ${label}✨`);
            })
        })();
    });
});



world.beforeEvents.itemUse.subscribe(data => {
    const player = data.source;
    if (!player || data.itemStack.typeId !== "kittys:crates_scoreboard") return;

    system.run(() => {
        const blockLook = player.getBlockFromViewDirection({ maxDistance: 10 })?.block;
        if (!blockLook) {
            player.sendMessage('§cPlease Look At A Block Near You!✨');
            return
        }
        const aboveLoc = { x: blockLook.x + 0.5, y: blockLook.y + 1, z: blockLook.z + 0.5 };
        const CratesLeaderboard = player.dimension.spawnEntity('kittys:hologram', aboveLoc);
        world.getDimension('overworld').runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
        CratesLeaderboard.setDynamicProperty('ObjectiveName', 'cratesopened');
        CratesLeaderboard.setDynamicProperty('PlayerCount', 10);
        CratesLeaderboard.addTag('kittysCrateLeaderboard');
        CratesLeaderboard.addTag('kittysLeaderboard');
    })
});
let titleToggle = 0;

system.runInterval(() => {
    for (const entity of world.getDimension('overworld').getEntities({ type: 'kittys:hologram', tags: ['kittysCrateLeaderboard'] })) {
        let obj = entity.getDynamicProperty('ObjectiveName');
        const playerCount = entity.getDynamicProperty('PlayerCount');
        let newTop = topLeaderboard(obj);
        const LeaderboardTitle1 = '§6Total Crates Players \n§eOpened:';
        const LeaderboardTitle2 = '§eTotal Crates Players \n§6Opened:';
        titleToggle = (titleToggle + 1) % 2; // Toggle between 0 and 1
        let currentTitle = titleToggle === 0 ? LeaderboardTitle1 : LeaderboardTitle2;

        /** @type {{name:string,score:number}[]} */
        const playerData = JSON.parse(entity.getDynamicProperty('LeaderboardData') ?? '[]');
        playerData.forEach(data => {
            if (newTop.some(v => v.name === data.name)) return;
            newTop.push({
                name: data.name,
                score: data.score
            })
        });
        while (newTop.length < playerCount) {
            newTop.push({
                name: `Offline Player ${newTop.length + 1}`,
                score: 0
            });
        }
        newTop.sort((a, b) => b.score - a.score);
        const slicedTop = newTop.slice(0, entity.getDynamicProperty('playercount'));
        slicedTop.forEach((data, index) => {
            const amount = data.score;
            let amtFormatted = amount.toString();
            if (amount >= 1000) {
                for (let i = 0; i < formatting.length; i++)
                    if (amount >= formatting[i].divider)
                        amtFormatted = (Math.floor((amount / formatting[i].divider) * 100) / 100) + formatting[i].suffix;
            }
            currentTitle += `\n§a${index + 1}§r. §b${data.name}§r: §c${amtFormatted}`;
        });
        entity.nameTag = currentTitle;
        entity.setDynamicProperty('LeaderboardData', JSON.stringify(newTop.slice(0, entity.getDynamicProperty('playercount') + 10)));
    }
}, 10);

function topLeaderboard(obj) {
    /** @type {{name:string,score:number}[]} */
    const scores = [];
    world.getPlayers().forEach(plr => {
        const playerScore = getScore(plr, obj);
        scores.push({ name: plr.name, score: playerScore });
    });
    scores.sort((a, b) => b.score - a.score);
    const topScores = scores.slice(0, 15);
    return topScores;
};

const objDB = {};
function getScore(target, objective) {
    try {
        return (objDB[objective] ??= world.scoreboard.getObjective(objective))?.getScore(target) ?? 0;
    } catch { return 0; }
};
//  ██████╗██████╗  █████╗ ████████╗███████╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║     ██████╔╝███████║   ██║   █████╗  
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝  
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝
//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleCrateChatcommand = SettingsDb.get('KittysCrateSettings:ToggleCrateChatcommand') || false;
    if (ToggleCrateChatcommand === 'true' || ToggleCrateChatcommand === true) {
        return;
    }
    const Crate_Command = SettingsDb.get('KittysCrateSettings:Crate_Command') || '!crate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${Crate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = SettingsDb.get('KittysCrateSettings:Crate_Command') || '!crate';
    const crateTimerValue = SettingsDb.get('KittysCrateSettings:CrateTimer') || '2';
    const ToggleCrateChatcommand = SettingsDb.get('KittysCrateSettings:ToggleCrateChatcommand') || false;
    if (ToggleCrateChatcommand === 'true' || ToggleCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await Admin_Main_Menu(data.sender);
        return;
    }

    async function checkItemAmount(player) {
        const component = player.getComponent("minecraft:inventory");
        const inventory = component.container;
        let itemAmount = 0;
        for (let i = 0; i < 36; i++) {
            let item = inventory.getItem(i);
            if (item?.typeId === "kittys:crate_key") {
                itemAmount += item.amount;
            }
        }
        return itemAmount;
    }

    const itemAmount = await checkItemAmount(data.sender);

    if (itemAmount >= 1) {
        system.run(() => {
            data.sender.runCommand(`clear "${data.sender.name}" kittys:crate_key 0 1`);
            data.sender.runCommand(`tag "${data.sender.name}" add keyed`);
            data.sender.sendMessage(`§c§lKey Used!✨`);
        });

        await data.sender.sendMessage('§6§lOpening Crate Menu!✨');
        const crateMenu = new ActionFormData()
            .title('!Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.sender, crateMenu);
            result.selection === 0 ? await Crate_Form(data.sender) : await closeCrateMenu(data.sender);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }

    const savedSettings = getSettingsDb();
    const ToggleCoolDownSystemValue = savedSettings.ToggleCoolDownSystemcommand === 'true';
    if (ToggleCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!cooldowns.has(player.name)) return false;

        const playerCooldown = cooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - cooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await Crate_Form(data.sender) : await closeCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function Crate_Form(player) {
    const crateTimerValue = SettingsDb.get('KittysCrateSettings:CrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!cooldowns.has(player.name)) return false;
        const cooldown = cooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner") ||
        player.hasTag("keyed")
    ) {
        const allPrizes = getPrizes();
        const randomPrize = get_RandomPrize(allPrizes);
        await givePlayer_Prize(player, randomPrize);
        await _Crate_Result_Form(player, randomPrize, allPrizes);
        const overworld = world.getDimension('overworld');
        await overworld.runCommand(`tag "${player.name}" remove keyed`)
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - cooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getPrizes();
    const randomPrize = get_RandomPrize(allPrizes);
    await givePlayer_Prize(player, randomPrize);
    await _Crate_Result_Form(player, randomPrize, allPrizes);

    cooldowns.set(player.name, Date.now());
}

const getPrizes = () => {
    const customPrizes = ItemsDb.getAll();
    if (customPrizes.length === 0) {
        return prizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(ItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function get_RandomPrize(prizes) {
    const totalWeight = prizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of prizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayer_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    await overworld.runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
    await overworld.runCommandAsync(`scoreboard players add "${player.name}" cratesopened 1`);
    const savedSettings = getSettingsDb();
    const ToggleOpenAnimationSystemValue = savedSettings.ToggleOpenAnimationcommand === 'true';
    if (ToggleOpenAnimationSystemValue === false) {
        await overworld.runCommand("function chest")
    }
    if (!player.hasTag("keyed")
    ) {
        setCooldown(player.name);
    }
    const ToggleSoundSystemValue = savedSettings.ToggleSoundcommand === 'true';
    if (ToggleSoundSystemValue === false) {
        const ToggleSoundFileValue = Number(savedSettings.ToggleSoundFilecommand);
        playCrateSound(ToggleSoundFileValue, player);
    }
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function _Crate_Result_Form(player, wonPrize, allPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleCrateAnnouncementcommand = SettingsDb.get('KittysCrateSettings:ToggleCrateAnnouncementcommand') || false;
    if (ToggleCrateAnnouncementcommand === 'false' || ToggleCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Crate!✨`);
    }
    allPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getSettingsDb = () => {
    const savedSettings = {
        CrateTimer: SettingsDb.get('KittysCrateSettings:CrateTimer') || '2',
        Crate_Command: SettingsDb.get('KittysCrateSettings:Crate_Command') || '!crate',
        ToggleCrateChatcommand: SettingsDb.get('KittysCrateSettings:ToggleCrateChatcommand') || false,
        ToggleCrateAnnouncementcommand: SettingsDb.get('KittysCrateSettings:ToggleCrateAnnouncementcommand') || false,
        ToggleCoolDownSystemcommand: SettingsDb.get('KittysCrateSettings:ToggleCoolDownSystemcommand') || false,
        ToggleOpenAnimationcommand: SettingsDb.get('KittysCrateSettings:ToggleOpenAnimationcommand') || false,
        ToggleSoundcommand: SettingsDb.get('KittysCrateSettings:ToggleSoundcommand') || false,
        ToggleSoundFilecommand: SettingsDb.get('KittysCrateSettings:ToggleSoundFilecommand') || '1',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function Admin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Crate')
        .body('Admin !Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await Crate_Form(sender);
        } else if (result.selection === 2) {
            await Admin_Crate_Settings_Main(sender);
        } else {
            await closeCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function Admin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove / Edit Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const savedSettings = getSettingsDb();
                const CrateTimerValue = Number(savedSettings.CrateTimer);
                const ToggleCrateChatcommandValue = savedSettings.ToggleCrateChatcommand === 'true' ? true : false;
                const ToggleCrateAnnouncementcommandValue = savedSettings.ToggleCrateAnnouncementcommand === 'true' ? true : false;
                const ToggleCoolDownSystemValue = savedSettings.ToggleCoolDownSystemcommand === 'true' ? true : false;
                const ToggleOpenAnimationValue = savedSettings.ToggleOpenAnimationcommand === 'true' ? true : false;
                const ToggleSoundValue = savedSettings.ToggleSoundcommand === 'true' ? true : false;
                const ToggleSoundFileValue = Number(savedSettings.ToggleSoundFilecommand)

                const form = new ModalFormData()
                    .title('Kittys Crate Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 48, 1, CrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!crate', savedSettings.Crate_Command)
                    .toggle(`Disable ${savedSettings.Crate_Command} Chat Command`, ToggleCrateChatcommandValue)
                    .toggle(`Disable !crate Chat Prize Announcement`, ToggleCrateAnnouncementcommandValue)
                    .toggle(`Disable Crate Cool Down System`, ToggleCoolDownSystemValue)
                    .toggle(`Disable Crate Open Animation`, ToggleOpenAnimationValue)
                    .toggle(`Disable Crate Sound`, ToggleSoundValue)
                    .slider(`Pick Crate Prize Sound✨`, 1, 12, 1, ToggleSoundFileValue)


                form.show(player).then(response => {
                    const [CrateTimer, Crate_Command, ToggleCrateChatcommand, ToggleCrateAnnouncementcommand, ToggleCoolDownSystemcommand, ToggleOpenAnimationcommand, ToggleSoundcommand, ToggleSoundFilecommand] = response.formValues;
                    if (!Crate_Command || typeof Crate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return Admin_Main_Menu(player);
                    }
                    SettingsDb.set('KittysCrateSettings:ToggleCrateChatcommand', `${ToggleCrateChatcommand}`);
                    SettingsDb.set('KittysCrateSettings:ToggleCrateAnnouncementcommand', `${ToggleCrateAnnouncementcommand}`);
                    SettingsDb.set('KittysCrateSettings:ToggleCoolDownSystemcommand', `${ToggleCoolDownSystemcommand}`);
                    SettingsDb.set('KittysCrateSettings:ToggleOpenAnimationcommand', `${ToggleOpenAnimationcommand}`);
                    SettingsDb.set('KittysCrateSettings:ToggleSoundcommand', `${ToggleSoundcommand}`);
                    SettingsDb.set('KittysCrateSettings:ToggleSoundFilecommand', `${ToggleSoundFilecommand}`);
                    SettingsDb.set('KittysCrateSettings:CrateTimer', `${CrateTimer}`);
                    SettingsDb.set('KittysCrateSettings:Crate_Command', Crate_Command);
                    return Admin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        ItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return Admin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allItemsDb = ItemsDb.getAll();
                    if (allItemsDb.length === 0) {
                        world.sendMessage("No Items to edit or remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Items Menu")
                            .body(`Select an Item to Edit or Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(ItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity;

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` ${itemKey} (x${itemQuantity}) \n (Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(`§o§1Last Page\n§3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allItemsDb.length) {
                            form.button(`§o§1Next Page\n§3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allItemsDb.length ? 1 : 0;

                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return Admin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }

                        const itemToEdit = allItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(ItemsDb.get(itemToEdit));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;

                        try {
                            let texturePath;
                            if (isitem.has(itemToEdit)) {
                                texturePath = `textures/items/${itemToEdit}`;
                            } else if (isblock.has(itemToEdit)) {
                                texturePath = `textures/blocks/${itemToEdit}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            const EditForm = new ActionFormData()
                                .title('Edit Item Menu')
                                .body(`Edit the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`)
                                .button("Edit Quantity✨", "textures/ui/icon_armor")
                                .button("Edit Rarity✨", "textures/ui/icon_armor")
                                .button("Edit ID✨", "textures/ui/icon_armor")
                                .button(`Remove Item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const editResponse = await EditForm.show(player);

                            if (editResponse.selection === 0) {
                                // Edit Item Quantity
                                const QuantityForm = new ModalFormData()
                                    .title("Edit Item Quantity")
                                    .slider(`Set new quantity for ${itemToEdit}✨`, 1, 64, 1, itemQuantity);
                                const quantityResponse = await QuantityForm.show(player);
                                const newQuantity = quantityResponse.formValues[0];

                                itemData.itemValue = newQuantity;
                                ItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Quantity updated for ${itemToEdit}: now x${newQuantity}✨`);

                            } else if (editResponse.selection === 1) {
                                // Edit Item Rarity
                                const RarityForm = new ModalFormData()
                                    .title("Edit Item Rarity")
                                    .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, itemData.rarity);
                                const rarityResponse = await RarityForm.show(player);
                                const newRarity = rarityResponse.formValues[0];

                                itemData.rarity = newRarity;
                                ItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Rarity updated for ${itemToEdit}: now ${newRarity}✨`);

                            } else if (editResponse.selection === 2) {
                                // Edit ID
                                const IdForm = new ModalFormData()
                                    .title("Edit Item ID")
                                    .textField(`Set new ID for ${itemToEdit}:`, "Enter new ID", `${itemToEdit}✨`);
                                const idResponse = await IdForm.show(player);
                                const newItemId = idResponse.formValues[0];

                                if (newItemId) {
                                    ItemsDb.del(itemToEdit); // Remove the old ID entry
                                    ItemsDb.set(newItemId, JSON.stringify(itemData)); // Save with new ID
                                    world.sendMessage(`Item ID updated: ${itemToEdit} → ${newItemId}✨`);
                                    return
                                } else {
                                    world.sendMessage("Invalid item ID.✨");
                                }
                            } else if (editResponse.selection === 3) {
                                // Confirm Removal
                                const ConfirmForm = new ActionFormData()
                                    .title('Confirm Item Removal')
                                    .body(`Are you sure you want to remove the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                    .button(`Remove: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`, texturePath)
                                    .button("Go Back✨", "textures/ui/icon_map");

                                const confirmResponse = await ConfirmForm.show(player);

                                if (confirmResponse.selection === 0) {
                                    ItemsDb.del(itemToEdit);
                                    world.sendMessage(`Item removed: ${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the edit menu.✨", error);
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function closeCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target?.typeId !== "kittys:chest") return;
    if (player && data.itemStack && data.itemStack.typeId === "kittys:removal_tool") {
        return;
    };
    const crateTimerValue = SettingsDb.get('KittysCrateSettings:CrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await Admin_Main_Menu(data.player);
        return;
    }
    if (player && data.itemStack && data.itemStack.typeId === "kittys:crate_key") {
        await data.player.sendMessage('§6§lOpening Crate Menu!✨');
        system.run(() => {
            target.dimension.runCommand(`clear "${player.name}" kittys:crate_key 0 1`)
            target.dimension.runCommand(`say §c§lKey Used!✨`);

            target.dimension.runCommand(`tag "${player.name}" add keyed`)
        });

        const crateMenu = new ActionFormData()
            .title('!Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.player, crateMenu);
            result.selection === 0 ? await Crate_Form(data.player) : await closeCrateMenu(data.player);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!cooldowns.has(player.name)) return false;
        const cooldown = cooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    const savedSettings = getSettingsDb();
    const ToggleCoolDownSystemValue = savedSettings.ToggleCoolDownSystemcommand === 'true';
    if (ToggleCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - cooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');

    const crateMenu = new ActionFormData()
        .title('!Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, crateMenu);
        result.selection === 0 ? await Crate_Form(data.player) : await closeCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});















// ██╗     ███████╗ ██████╗ ███████╗███╗   ██╗██████╗  █████╗ ██████╗ ██╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗
// ██║     ██╔════╝██╔════╝ ██╔════╝████╗  ██║██╔══██╗██╔══██╗██╔══██╗╚██╗ ██╔╝    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║     █████╗  ██║  ███╗█████╗  ██╔██╗ ██║██║  ██║███████║██████╔╝ ╚████╔╝     ██║     ██████╔╝███████║   ██║   █████╗  
// ██║     ██╔══╝  ██║   ██║██╔══╝  ██║╚██╗██║██║  ██║██╔══██║██╔══██╗  ╚██╔╝      ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝  
// ███████╗███████╗╚██████╔╝███████╗██║ ╚████║██████╔╝██║  ██║██║  ██║   ██║       ╚██████╗██║  ██║██║  ██║   ██║   ███████╗
// ╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝  ╚═══╝╚═════╝ ╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝                                                                                                                    
//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleLegendaryCrateChatcommand = LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand') || false;
    if (ToggleLegendaryCrateChatcommand === 'true' || ToggleLegendaryCrateChatcommand === true) {
        return;
    }
    const LegendaryCrate_Command = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrate_Command') || '!legendarycrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${LegendaryCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrate_Command') || '!legendarycrate';
    const crateTimerValue = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrateTimer') || '2';
    const ToggleLegendaryCrateChatcommand = LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand') || false;
    if (ToggleLegendaryCrateChatcommand === 'true' || ToggleLegendaryCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await LegendaryAdmin_Main_Menu(data.sender);
        return;
    }

    async function checkItemAmount(player) {
        const component = player.getComponent("minecraft:inventory");
        const inventory = component.container;
        let itemAmount = 0;
        for (let i = 0; i < 36; i++) {
            let item = inventory.getItem(i);
            if (item?.typeId === "kittys:legendarycrate_key") {
                itemAmount += item.amount;
            }
        }
        return itemAmount;
    }

    const itemAmount = await checkItemAmount(data.sender);

    if (itemAmount >= 1) {
        system.run(() => {
            data.sender.runCommand(`clear "${data.sender.name}" kittys:legendarycrate_key 0 1`);
            data.sender.runCommand(`tag "${data.sender.name}" add Legendarykeyed`);
            data.sender.sendMessage(`§c§lLegendary Key Used!✨`);
        });

        await data.sender.sendMessage('§6§lOpening Crate Menu!✨');
        const LegendarycrateMenu = new ActionFormData()
            .title('!Legendary Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.sender, LegendarycrateMenu);
            result.selection === 0 ? await LegendaryCrate_Form(data.sender) : await LegendarycloseCrateMenu(data.sender);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }

    const LegendarysavedSettings = getLegendarySettingsDb();
    const ToggleLegendaryCoolDownSystemValue = LegendarysavedSettings.ToggleLegendaryCoolDownSystemcommand === 'true';
    if (ToggleLegendaryCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Legendarycooldowns.has(player.name)) return false;

        const playerCooldown = Legendarycooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Legendarycooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the legendary crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Legendary Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await LegendaryCrate_Form(data.sender) : await LegendarycloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function LegendaryCrate_Form(player) {
    const crateTimerValue = LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Legendarycooldowns.has(player.name)) return false;
        const cooldown = Legendarycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner") ||
        player.hasTag("Legendarykeyed")
    ) {
        const allPrizes = getLegendaryPrizes();
        const randomPrize = getLegendary_RandomPrize(allPrizes);
        await givePlayerLegendary_Prize(player, randomPrize);
        await Legendary_Crate_Result_Form(player, randomPrize, allPrizes);
        const overworld = world.getDimension('overworld');
        await overworld.runCommand(`tag "${player.name}" remove Legendarykeyed`)
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Legendarycooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the legendary crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getLegendaryPrizes();
    const randomPrize = getLegendary_RandomPrize(allPrizes);
    await givePlayerLegendary_Prize(player, randomPrize);
    await Legendary_Crate_Result_Form(player, randomPrize, allPrizes);

    Legendarycooldowns.set(player.name, Date.now());
}

const getLegendaryPrizes = () => {
    const customPrizes = LegendaryItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Legendaryprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(LegendaryItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getLegendary_RandomPrize(Legendaryprizes) {
    const totalWeight = Legendaryprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Legendaryprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerLegendary_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    await overworld.runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
    await overworld.runCommandAsync(`scoreboard players add "${player.name}" cratesopened 1`);
    const LegendarysavedSettings = getLegendarySettingsDb();
    const ToggleLegendaryOpenAnimationSystemValue = LegendarysavedSettings.ToggleLegendaryOpenAnimationcommand === 'true';
    if (ToggleLegendaryOpenAnimationSystemValue === false) {
        await overworld.runCommand("function legendarychest")
    }
    if (!player.hasTag("Legendarykeyed")
    ) {
        LegendarysetCooldown(player.name);
    }
    const ToggleLegendarySoundSystemValue = LegendarysavedSettings.ToggleLegendarySoundcommand === 'true';
    if (ToggleLegendarySoundSystemValue === false) {
        const ToggleLegendarySoundFileValue = Number(LegendarysavedSettings.ToggleLegendarySoundFilecommand);
        playCrateSound(ToggleLegendarySoundFileValue, player);
    }
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Legendary_Crate_Result_Form(player, wonPrize, LegendaryallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Legendary Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleLegendaryCrateAnnouncementcommand = LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateAnnouncementcommand') || false;
    if (ToggleLegendaryCrateAnnouncementcommand === 'false' || ToggleLegendaryCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Legendary Crate!✨`);
    }
    LegendaryallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getLegendarySettingsDb = () => {
    const savedSettings = {
        LegendaryCrateTimer: LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrateTimer') || '2',
        LegendaryCrate_Command: LegendarySettingsDb.get('LegendaryKittysCrateSettings:LegendaryCrate_Command') || '!legendarycrate',
        ToggleLegendaryCrateChatcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand') || false,
        ToggleLegendaryCrateAnnouncementcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCrateAnnouncementcommand') || false,
        ToggleLegendaryCoolDownSystemcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryCoolDownSystemcommand') || false,
        ToggleLegendaryOpenAnimationcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendaryOpenAnimationcommand') || false,
        ToggleLegendarySoundcommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendarySoundcommand') || false,
        ToggleLegendarySoundFilecommand: LegendarySettingsDb.get('LegendaryKittysCrateSettings:ToggleLegendarySoundFilecommand') || '2',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function LegendaryAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Legendary Crate')
        .body('Admin !Legendary Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await LegendaryCrate_Form(sender);
        } else if (result.selection === 2) {
            await LegendaryAdmin_Crate_Settings_Main(sender);
        } else {
            await LegendarycloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function LegendaryAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Legendary Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove / Edit Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const LegendarysavedSettings = getLegendarySettingsDb();
                const LegendaryCrateTimerValue = Number(LegendarysavedSettings.LegendaryCrateTimer);
                const ToggleLegendaryCrateChatcommandValue = LegendarysavedSettings.ToggleLegendaryCrateChatcommand === 'true' ? true : false;
                const ToggleLegendaryCrateAnnouncementcommandValue = LegendarysavedSettings.ToggleLegendaryCrateAnnouncementcommand === 'true' ? true : false;
                const ToggleLegendaryCoolDownSystemValue = LegendarysavedSettings.ToggleLegendaryCoolDownSystemcommand === 'true' ? true : false;
                const ToggleLegendaryOpenAnimationValue = LegendarysavedSettings.ToggleLegendaryOpenAnimationcommand === 'true' ? true : false;
                const ToggleLegendarySoundValue = LegendarysavedSettings.ToggleLegendarySoundcommand === 'true' ? true : false;
                const ToggleLegendarySoundFileValue = Number(LegendarysavedSettings.ToggleLegendarySoundFilecommand)

                const form = new ModalFormData()
                    .title('Kittys Crate Legendary Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 48, 1, LegendaryCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!legendarycrate', LegendarysavedSettings.LegendaryCrate_Command)
                    .toggle(`Disable ${LegendarysavedSettings.LegendaryCrate_Command} Chat Command`, ToggleLegendaryCrateChatcommandValue)
                    .toggle(`Disable !legendarycrate Chat Prize Announcement`, ToggleLegendaryCrateAnnouncementcommandValue)
                    .toggle(`Disable Legendary Crate Cool Down System`, ToggleLegendaryCoolDownSystemValue)
                    .toggle(`Disable Legendary Crate Open Animation`, ToggleLegendaryOpenAnimationValue)
                    .toggle(`Disable Legendary Crate Sound`, ToggleLegendarySoundValue)
                    .slider(`Pick Legendary Crate Prize Sound✨`, 1, 12, 1, ToggleLegendarySoundFileValue)


                form.show(player).then(response => {
                    const [LegendaryCrateTimer, LegendaryCrate_Command, ToggleLegendaryCrateChatcommand, ToggleLegendaryCrateAnnouncementcommand, ToggleLegendaryCoolDownSystemcommand, ToggleLegendaryOpenAnimationcommand, ToggleLegendarySoundcommand, ToggleLegendarySoundFilecommand] = response.formValues;

                    if (!LegendaryCrate_Command || typeof LegendaryCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return LegendaryAdmin_Main_Menu(player);
                    }
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendaryCrateChatcommand', `${ToggleLegendaryCrateChatcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendaryCrateAnnouncementcommand', `${ToggleLegendaryCrateAnnouncementcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendaryCoolDownSystemcommand', `${ToggleLegendaryCoolDownSystemcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendaryOpenAnimationcommand', `${ToggleLegendaryOpenAnimationcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendarySoundcommand', `${ToggleLegendarySoundcommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:ToggleLegendarySoundFilecommand', `${ToggleLegendarySoundFilecommand}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:LegendaryCrateTimer', `${LegendaryCrateTimer}`);
                    LegendarySettingsDb.set('LegendaryKittysCrateSettings:LegendaryCrate_Command', LegendaryCrate_Command);
                    return LegendaryAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Legendary Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        LegendaryItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return LegendaryAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allLegendaryItemsDb = LegendaryItemsDb.getAll();
                    if (allLegendaryItemsDb.length === 0) {
                        world.sendMessage("No Items to edit or remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allLegendaryItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Legendary Items Menu")
                            .body(`Select an Item to Edit or Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allLegendaryItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(LegendaryItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity;

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` ${itemKey} (x${itemQuantity}) \n (Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(`§o§1Last Page\n§3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allLegendaryItemsDb.length) {
                            form.button(`§o§1Next Page\n§3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allLegendaryItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allLegendaryItemsDb.length ? 1 : 0;

                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return LegendaryAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allLegendaryItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }

                        const itemToEdit = allLegendaryItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(LegendaryItemsDb.get(itemToEdit));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;

                        try {
                            let texturePath;
                            if (isitem.has(itemToEdit)) {
                                texturePath = `textures/items/${itemToEdit}`;
                            } else if (isblock.has(itemToEdit)) {
                                texturePath = `textures/blocks/${itemToEdit}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            const EditForm = new ActionFormData()
                                .title('Legendary Edit Item Menu')
                                .body(`Edit the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`)
                                .button("Edit Quantity✨", "textures/ui/icon_armor")
                                .button("Edit Rarity✨", "textures/ui/icon_armor")
                                .button("Edit ID✨", "textures/ui/icon_armor")
                                .button(`Remove Item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const editResponse = await EditForm.show(player);

                            if (editResponse.selection === 0) {
                                // Edit Item Quantity
                                const QuantityForm = new ModalFormData()
                                    .title("Edit Item Quantity")
                                    .slider(`Set new quantity for ${itemToEdit}✨`, 1, 64, 1, itemQuantity);
                                const quantityResponse = await QuantityForm.show(player);
                                const newQuantity = quantityResponse.formValues[0];

                                itemData.itemValue = newQuantity;
                                LegendaryItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Quantity updated for ${itemToEdit}: now x${newQuantity}✨`);

                            } else if (editResponse.selection === 1) {
                                // Edit Item Rarity
                                const RarityForm = new ModalFormData()
                                    .title("Edit Item Rarity")
                                    .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, itemData.rarity);
                                const rarityResponse = await RarityForm.show(player);
                                const newRarity = rarityResponse.formValues[0];

                                itemData.rarity = newRarity;
                                LegendaryItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Rarity updated for ${itemToEdit}: now ${newRarity}✨`);

                            } else if (editResponse.selection === 2) {
                                // Edit ID
                                const IdForm = new ModalFormData()
                                    .title("Edit Item ID")
                                    .textField(`Set new ID for ${itemToEdit}:`, "Enter new ID", `${itemToEdit}✨`);
                                const idResponse = await IdForm.show(player);
                                const newItemId = idResponse.formValues[0];

                                if (newItemId) {
                                    LegendaryItemsDb.del(itemToEdit); // Remove the old ID entry
                                    LegendaryItemsDb.set(newItemId, JSON.stringify(itemData)); // Save with new ID
                                    world.sendMessage(`Item ID updated: ${itemToEdit} → ${newItemId}✨`);
                                    return
                                } else {
                                    world.sendMessage("Invalid item ID.✨");
                                }
                            } else if (editResponse.selection === 3) {
                                // Confirm Removal
                                const ConfirmForm = new ActionFormData()
                                    .title('Legendary Confirm Item Removal')
                                    .body(`Are you sure you want to remove the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                    .button(`Remove: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`, texturePath)
                                    .button("Go Back✨", "textures/ui/icon_map");

                                const confirmResponse = await ConfirmForm.show(player);

                                if (confirmResponse.selection === 0) {
                                    LegendaryItemsDb.del(itemToEdit);
                                    world.sendMessage(`Item removed: ${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the edit menu.✨", error);
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function LegendarycloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:legendarychest") return
    if (player && data.itemStack && data.itemStack.typeId === "kittys:removal_tool") {
        return;
    };
    const LegendarycrateTimerValue = LegendarySettingsDb.get('KittysLegendaryCrateSettings:LegendaryCrateTimer') || '2';
    const LegendarycrateTimerHours = Number(LegendarycrateTimerValue);
    const cooldownTime = LegendarycrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await LegendaryAdmin_Main_Menu(data.player);
        return;
    }
    if (player && data.itemStack && data.itemStack.typeId === "kittys:legendarycrate_key") {
        await data.player.sendMessage('§6§lOpening Crate Menu!✨');
        system.run(() => {
            target.dimension.runCommand(`clear "${player.name}" kittys:legendarycrate_key 0 1`)
            target.dimension.runCommand(`say §c§lLegendary Key Used!✨`);

            target.dimension.runCommand(`tag "${player.name}" add Legendarykeyed`)
        });

        const LegendarycrateMenu = new ActionFormData()
            .title('!Legendary Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.player, LegendarycrateMenu);
            result.selection === 0 ? await LegendaryCrate_Form(data.player) : await LegendarycloseCrateMenu(data.player);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Legendarycooldowns.has(player.name)) return false;
        const cooldown = Legendarycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    const LegendarysavedSettings = getLegendarySettingsDb();
    const ToggleLegendaryCoolDownSystemValue = LegendarysavedSettings.ToggleLegendaryCoolDownSystemcommand === 'true';
    if (ToggleLegendaryCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Legendarycooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the legendary crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');

    const LegendarycrateMenu = new ActionFormData()
        .title('!Legendary Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, LegendarycrateMenu);
        result.selection === 0 ? await LegendaryCrate_Form(data.player) : await LegendarycloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});













//  ██████╗  █████╗ ██╗      █████╗ ██╗  ██╗██╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗
// ██╔════╝ ██╔══██╗██║     ██╔══██╗╚██╗██╔╝╚██╗ ██╔╝    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝
// ██║  ███╗███████║██║     ███████║ ╚███╔╝  ╚████╔╝     ██║     ██████╔╝███████║   ██║   █████╗  
// ██║   ██║██╔══██║██║     ██╔══██║ ██╔██╗   ╚██╔╝      ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝  
// ╚██████╔╝██║  ██║███████╗██║  ██║██╔╝ ██╗   ██║       ╚██████╗██║  ██║██║  ██║   ██║   ███████╗
//  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝                                                                                       
//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleGalaxyCrateChatcommand = GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand') || false;
    if (ToggleGalaxyCrateChatcommand === 'true' || ToggleGalaxyCrateChatcommand === true) {
        return;
    }
    const GalaxyCrate_Command = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrate_Command') || '!galaxycrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${GalaxyCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrate_Command') || '!galaxycrate';
    const crateTimerValue = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrateTimer') || '2';
    const ToggleGalaxyCrateChatcommand = GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand') || false;
    if (ToggleGalaxyCrateChatcommand === 'true' || ToggleGalaxyCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await GalaxyAdmin_Main_Menu(data.sender);
        return;
    }

    async function checkItemAmount(player) {
        const component = player.getComponent("minecraft:inventory");
        const inventory = component.container;
        let itemAmount = 0;
        for (let i = 0; i < 36; i++) {
            let item = inventory.getItem(i);
            if (item?.typeId === "kittys:galaxycrate_key") {
                itemAmount += item.amount;
            }
        }
        return itemAmount;
    }

    const itemAmount = await checkItemAmount(data.sender);

    if (itemAmount >= 1) {
        system.run(() => {
            data.sender.runCommand(`clear "${data.sender.name}" kittys:galaxycrate_key 0 1`);
            data.sender.runCommand(`tag "${data.sender.name}" add Galaxykeyed`);
            data.sender.sendMessage(`§c§lGalaxy Key Used!✨`);
        });

        await data.sender.sendMessage('§6§lOpening Crate Menu!✨');
        const GalaxycrateMenu = new ActionFormData()
            .title('!Galaxy Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.sender, GalaxycrateMenu);
            result.selection === 0 ? await GalaxyCrate_Form(data.sender) : await GalaxycloseCrateMenu(data.sender);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }

    const GalaxysavedSettings = getGalaxySettingsDb();
    const ToggleGalaxyCoolDownSystemValue = GalaxysavedSettings.ToggleGalaxyCoolDownSystemcommand === 'true';
    if (ToggleGalaxyCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Galaxycooldowns.has(player.name)) return false;

        const playerCooldown = Galaxycooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Galaxycooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the galaxy crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Galaxy Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await GalaxyCrate_Form(data.sender) : await GalaxycloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function GalaxyCrate_Form(player) {
    const crateTimerValue = GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Galaxycooldowns.has(player.name)) return false;
        const cooldown = Galaxycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner") ||
        player.hasTag("Galaxykeyed")
    ) {
        const allPrizes = getGalaxyPrizes();
        const randomPrize = getGalaxy_RandomPrize(allPrizes);
        await givePlayerGalaxy_Prize(player, randomPrize);
        await Galaxy_Crate_Result_Form(player, randomPrize, allPrizes);
        const overworld = world.getDimension('overworld');
        await overworld.runCommand(`tag "${player.name}" remove Galaxykeyed`)
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Galaxycooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the galaxy crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getGalaxyPrizes();
    const randomPrize = getGalaxy_RandomPrize(allPrizes);
    await givePlayerGalaxy_Prize(player, randomPrize);
    await Galaxy_Crate_Result_Form(player, randomPrize, allPrizes);

    Galaxycooldowns.set(player.name, Date.now());
}

const getGalaxyPrizes = () => {
    const customPrizes = GalaxyItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Galaxyprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(GalaxyItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getGalaxy_RandomPrize(Galaxyprizes) {
    const totalWeight = Galaxyprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Galaxyprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerGalaxy_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    await overworld.runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
    await overworld.runCommandAsync(`scoreboard players add "${player.name}" cratesopened 1`);
    const GalaxysavedSettings = getGalaxySettingsDb();
    const ToggleGalaxyOpenAnimationSystemValue = GalaxysavedSettings.ToggleGalaxyOpenAnimationcommand === 'true';
    if (ToggleGalaxyOpenAnimationSystemValue === false) {
        await overworld.runCommand("function galaxychest")
    }
    if (!player.hasTag("Galaxykeyed")
    ) {
        GalaxysetCooldown(player.name);
    }
    const ToggleGalaxySoundSystemValue = GalaxysavedSettings.ToggleGalaxySoundcommand === 'true';
    if (ToggleGalaxySoundSystemValue === false) {
        const ToggleGalaxySoundFileValue = Number(GalaxysavedSettings.ToggleGalaxySoundFilecommand);
        playCrateSound(ToggleGalaxySoundFileValue, player);
    }
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Galaxy_Crate_Result_Form(player, wonPrize, GalaxyallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Galaxy Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleGalaxyCrateAnnouncementcommand = GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateAnnouncementcommand') || false;
    if (ToggleGalaxyCrateAnnouncementcommand === 'false' || ToggleGalaxyCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Galaxy Crate!✨`);
    }
    GalaxyallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getGalaxySettingsDb = () => {
    const savedSettings = {
        GalaxyCrateTimer: GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrateTimer') || '2',
        GalaxyCrate_Command: GalaxySettingsDb.get('GalaxyKittysCrateSettings:GalaxyCrate_Command') || '!galaxycrate',
        ToggleGalaxyCrateChatcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand') || false,
        ToggleGalaxyCrateAnnouncementcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCrateAnnouncementcommand') || false,
        ToggleGalaxyCoolDownSystemcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyCoolDownSystemcommand') || false,
        ToggleGalaxyOpenAnimationcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxyOpenAnimationcommand') || false,
        ToggleGalaxySoundcommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxySoundcommand') || false,
        ToggleGalaxySoundFilecommand: GalaxySettingsDb.get('GalaxyKittysCrateSettings:ToggleGalaxySoundFilecommand') || '3',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function GalaxyAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Galaxy Crate')
        .body('Admin !Galaxy Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await GalaxyCrate_Form(sender);
        } else if (result.selection === 2) {
            await GalaxyAdmin_Crate_Settings_Main(sender);
        } else {
            await GalaxycloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function GalaxyAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Galaxy Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove / Edit Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const GalaxysavedSettings = getGalaxySettingsDb();
                const GalaxyCrateTimerValue = Number(GalaxysavedSettings.GalaxyCrateTimer);
                const ToggleGalaxyCrateChatcommandValue = GalaxysavedSettings.ToggleGalaxyCrateChatcommand === 'true' ? true : false;
                const ToggleGalaxyCrateAnnouncementcommandValue = GalaxysavedSettings.ToggleGalaxyCrateAnnouncementcommand === 'true' ? true : false;
                const ToggleGalaxyCoolDownSystemValue = GalaxysavedSettings.ToggleGalaxyCoolDownSystemcommand === 'true' ? true : false;
                const ToggleGalaxyOpenAnimationValue = GalaxysavedSettings.ToggleGalaxyOpenAnimationcommand === 'true' ? true : false;
                const ToggleGalaxySoundValue = GalaxysavedSettings.ToggleGalaxySoundcommand === 'true' ? true : false;
                const ToggleGalaxySoundFileValue = Number(GalaxysavedSettings.ToggleGalaxySoundFilecommand)

                const form = new ModalFormData()
                    .title('Kittys Crate Galaxy Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 48, 1, GalaxyCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!galaxycrate', GalaxysavedSettings.GalaxyCrate_Command)
                    .toggle(`Disable ${GalaxysavedSettings.GalaxyCrate_Command} Chat Command`, ToggleGalaxyCrateChatcommandValue)
                    .toggle(`Disable !galaxycrate Chat Prize Announcement`, ToggleGalaxyCrateAnnouncementcommandValue)
                    .toggle(`Disable Galaxy Crate Cool Down System`, ToggleGalaxyCoolDownSystemValue)
                    .toggle(`Disable Galaxy Crate Open Animation`, ToggleGalaxyOpenAnimationValue)
                    .toggle(`Disable Galaxy Crate Sound`, ToggleGalaxySoundValue)
                    .slider(`Pick Galaxy Crate Prize Sound✨`, 1, 12, 1, ToggleGalaxySoundFileValue)


                form.show(player).then(response => {
                    const [GalaxyCrateTimer, GalaxyCrate_Command, ToggleGalaxyCrateChatcommand, ToggleGalaxyCrateAnnouncementcommand, ToggleGalaxyCoolDownSystemcommand, ToggleGalaxyOpenAnimationcommand, ToggleGalaxySoundcommand, ToggleGalaxySoundFilecommand] = response.formValues;
                    if (!GalaxyCrate_Command || typeof GalaxyCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return GalaxyAdmin_Main_Menu(player);
                    }
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxyCrateChatcommand', `${ToggleGalaxyCrateChatcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxyCrateAnnouncementcommand', `${ToggleGalaxyCrateAnnouncementcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxyCoolDownSystemcommand', `${ToggleGalaxyCoolDownSystemcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxyOpenAnimationcommand', `${ToggleGalaxyOpenAnimationcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxySoundcommand', `${ToggleGalaxySoundcommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:ToggleGalaxySoundFilecommand', `${ToggleGalaxySoundFilecommand}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:GalaxyCrateTimer', `${GalaxyCrateTimer}`);
                    GalaxySettingsDb.set('GalaxyKittysCrateSettings:GalaxyCrate_Command', GalaxyCrate_Command);
                    return GalaxyAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Galaxy Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        GalaxyItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return GalaxyAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allGalaxyItemsDb = GalaxyItemsDb.getAll();
                    if (allGalaxyItemsDb.length === 0) {
                        world.sendMessage("No Items to edit or remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allGalaxyItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Galaxy Items Menu")
                            .body(`Select an Item to Edit or Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allGalaxyItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(GalaxyItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity;

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` ${itemKey} (x${itemQuantity}) \n (Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(`§o§1Last Page\n§3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allGalaxyItemsDb.length) {
                            form.button(`§o§1Next Page\n§3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allGalaxyItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allGalaxyItemsDb.length ? 1 : 0;

                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return GalaxyAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allGalaxyItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }

                        const itemToEdit = allGalaxyItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(GalaxyItemsDb.get(itemToEdit));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;

                        try {
                            let texturePath;
                            if (isitem.has(itemToEdit)) {
                                texturePath = `textures/items/${itemToEdit}`;
                            } else if (isblock.has(itemToEdit)) {
                                texturePath = `textures/blocks/${itemToEdit}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            const EditForm = new ActionFormData()
                                .title('Galaxy Edit Item Menu')
                                .body(`Edit the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`)
                                .button("Edit Quantity✨", "textures/ui/icon_armor")
                                .button("Edit Rarity✨", "textures/ui/icon_armor")
                                .button("Edit ID✨", "textures/ui/icon_armor")
                                .button(`Remove Item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const editResponse = await EditForm.show(player);

                            if (editResponse.selection === 0) {
                                // Edit Item Quantity
                                const QuantityForm = new ModalFormData()
                                    .title("Edit Item Quantity")
                                    .slider(`Set new quantity for ${itemToEdit}✨`, 1, 64, 1, itemQuantity);
                                const quantityResponse = await QuantityForm.show(player);
                                const newQuantity = quantityResponse.formValues[0];

                                itemData.itemValue = newQuantity;
                                GalaxyItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Quantity updated for ${itemToEdit}: now x${newQuantity}✨`);

                            } else if (editResponse.selection === 1) {
                                // Edit Item Rarity
                                const RarityForm = new ModalFormData()
                                    .title("Edit Item Rarity")
                                    .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, itemData.rarity);
                                const rarityResponse = await RarityForm.show(player);
                                const newRarity = rarityResponse.formValues[0];

                                itemData.rarity = newRarity;
                                GalaxyItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Rarity updated for ${itemToEdit}: now ${newRarity}✨`);

                            } else if (editResponse.selection === 2) {
                                // Edit ID
                                const IdForm = new ModalFormData()
                                    .title("Edit Item ID")
                                    .textField(`Set new ID for ${itemToEdit}:`, "Enter new ID", `${itemToEdit}✨`);
                                const idResponse = await IdForm.show(player);
                                const newItemId = idResponse.formValues[0];

                                if (newItemId) {
                                    GalaxyItemsDb.del(itemToEdit); // Remove the old ID entry
                                    GalaxyItemsDb.set(newItemId, JSON.stringify(itemData)); // Save with new ID
                                    world.sendMessage(`Item ID updated: ${itemToEdit} → ${newItemId}✨`);
                                    return
                                } else {
                                    world.sendMessage("Invalid item ID.✨");
                                }
                            } else if (editResponse.selection === 3) {
                                // Confirm Removal
                                const ConfirmForm = new ActionFormData()
                                    .title('Galaxy Confirm Item Removal')
                                    .body(`Are you sure you want to remove the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                    .button(`Remove: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`, texturePath)
                                    .button("Go Back✨", "textures/ui/icon_map");

                                const confirmResponse = await ConfirmForm.show(player);

                                if (confirmResponse.selection === 0) {
                                    GalaxyItemsDb.del(itemToEdit);
                                    world.sendMessage(`Item removed: ${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the edit menu.✨", error);
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function GalaxycloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:galaxychest") return
    if (player && data.itemStack && data.itemStack.typeId === "kittys:removal_tool") {
        return;
    };
    const GalaxycrateTimerValue = GalaxySettingsDb.get('KittysGalaxyCrateSettings:GalaxyCrateTimer') || '2';
    const GalaxycrateTimerHours = Number(GalaxycrateTimerValue);
    const cooldownTime = GalaxycrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await GalaxyAdmin_Main_Menu(data.player);
        return;
    }
    if (player && data.itemStack && data.itemStack.typeId === "kittys:galaxycrate_key") {
        await data.player.sendMessage('§6§lOpening Crate Menu!✨');
        system.run(() => {
            target.dimension.runCommand(`clear "${player.name}" kittys:galaxycrate_key 0 1`)
            target.dimension.runCommand(`say §c§lGalaxy Key Used!✨`);

            target.dimension.runCommand(`tag "${player.name}" add Galaxykeyed`)
        });

        const GalaxycrateMenu = new ActionFormData()
            .title('!Galaxy Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.player, GalaxycrateMenu);
            result.selection === 0 ? await GalaxyCrate_Form(data.player) : await GalaxycloseCrateMenu(data.player);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Galaxycooldowns.has(player.name)) return false;
        const cooldown = Galaxycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    const GalaxysavedSettings = getGalaxySettingsDb();
    const ToggleGalaxyCoolDownSystemValue = GalaxysavedSettings.ToggleGalaxyCoolDownSystemcommand === 'true';
    if (ToggleGalaxyCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Galaxycooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the galaxy crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');

    const GalaxycrateMenu = new ActionFormData()
        .title('!Galaxy Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, GalaxycrateMenu);
        result.selection === 0 ? await GalaxyCrate_Form(data.player) : await GalaxycloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});















//  ██████╗██████╗  █████╗ ███████╗██╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗        
// ██╔════╝██╔══██╗██╔══██╗╚══███╔╝╚██╗ ██╔╝    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝        
// ██║     ██████╔╝███████║  ███╔╝  ╚████╔╝     ██║     ██████╔╝███████║   ██║   █████╗          
// ██║     ██╔══██╗██╔══██║ ███╔╝    ╚██╔╝      ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝          
// ╚██████╗██║  ██║██║  ██║███████╗   ██║       ╚██████╗██║  ██║██║  ██║   ██║   ███████╗        
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝╚══════╝   ╚═╝        ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝                                                                                                                                                                                                             
//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleCrazyCrateChatcommand = CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyCrateChatcommand') || false;
    if (ToggleCrazyCrateChatcommand === 'true' || ToggleCrazyCrateChatcommand === true) {
        return;
    }
    const CrazyCrate_Command = CrazySettingsDb.get('CrazyKittysCrateSettings:CrazyCrate_Command') || '!crazycrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${CrazyCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = CrazySettingsDb.get('CrazyKittysCrateSettings:CrazyCrate_Command') || '!crazycrate';
    const crateTimerValue = CrazySettingsDb.get('CrazyKittysCrateSettings:CrazyCrateTimer') || '2';
    const ToggleCrazyCrateChatcommand = CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyCrateChatcommand') || false;
    if (ToggleCrazyCrateChatcommand === 'true' || ToggleCrazyCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await CrazyAdmin_Main_Menu(data.sender);
        return;
    }

    async function checkItemAmount(player) {
        const component = player.getComponent("minecraft:inventory");
        const inventory = component.container;
        let itemAmount = 0;
        for (let i = 0; i < 36; i++) {
            let item = inventory.getItem(i);
            if (item?.typeId === "kittys:crazycrate_key") {
                itemAmount += item.amount;
            }
        }
        return itemAmount;
    }

    const itemAmount = await checkItemAmount(data.sender);

    if (itemAmount >= 1) {
        system.run(() => {
            data.sender.runCommand(`clear "${data.sender.name}" kittys:crazycrate_key 0 1`);
            data.sender.runCommand(`tag "${data.sender.name}" add Crazykeyed`);
            data.sender.sendMessage(`§c§lCrazy Key Used!✨`);
        });

        await data.sender.sendMessage('§6§lOpening Crate Menu!✨');
        const CrazycrateMenu = new ActionFormData()
            .title('!Crazy Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.sender, CrazycrateMenu);
            result.selection === 0 ? await CrazyCrate_Form(data.sender) : await CrazycloseCrateMenu(data.sender);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }

    const CrazysavedSettings = getCrazySettingsDb();
    const ToggleCrazyCoolDownSystemValue = CrazysavedSettings.ToggleCrazyCoolDownSystemcommand === 'true';
    if (ToggleCrazyCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Crazycooldowns.has(player.name)) return false;

        const playerCooldown = Crazycooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Crazycooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crazy crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Crazy Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await CrazyCrate_Form(data.sender) : await CrazycloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function CrazyCrate_Form(player) {
    const crateTimerValue = CrazySettingsDb.get('CrazyKittysCrateSettings:CrazyCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Crazycooldowns.has(player.name)) return false;
        const cooldown = Crazycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner") ||
        player.hasTag("Crazykeyed")
    ) {
        const allPrizes = getCrazyPrizes();
        const randomPrize = getCrazy_RandomPrize(allPrizes);
        await givePlayerCrazy_Prize(player, randomPrize);
        await Crazy_Crate_Result_Form(player, randomPrize, allPrizes);
        const overworld = world.getDimension('overworld');
        await overworld.runCommand(`tag "${player.name}" remove Crazykeyed`)
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Crazycooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crazy crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getCrazyPrizes();
    const randomPrize = getCrazy_RandomPrize(allPrizes);
    await givePlayerCrazy_Prize(player, randomPrize);
    await Crazy_Crate_Result_Form(player, randomPrize, allPrizes);

    Crazycooldowns.set(player.name, Date.now());
}

const getCrazyPrizes = () => {
    const customPrizes = CrazyItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Crazyprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(CrazyItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getCrazy_RandomPrize(Crazyprizes) {
    const totalWeight = Crazyprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Crazyprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerCrazy_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    await overworld.runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
    await overworld.runCommandAsync(`scoreboard players add "${player.name}" cratesopened 1`);
    const CrazysavedSettings = getCrazySettingsDb();
    const ToggleCrazyOpenAnimationSystemValue = CrazysavedSettings.ToggleCrazyOpenAnimationcommand === 'true';
    if (ToggleCrazyOpenAnimationSystemValue === false) {
        await overworld.runCommand("function crazychest")
    }
    if (!player.hasTag("Crazykeyed")
    ) {
        CrazysetCooldown(player.name);
    }
    const ToggleCrazySoundSystemValue = CrazysavedSettings.ToggleCrazySoundcommand === 'true';
    if (ToggleCrazySoundSystemValue === false) {
        const ToggleCrazySoundFileValue = Number(CrazysavedSettings.ToggleCrazySoundFilecommand);
        playCrateSound(ToggleCrazySoundFileValue, player);
    }
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Crazy_Crate_Result_Form(player, wonPrize, CrazyallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Crazy Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleCrazyCrateAnnouncementcommand = CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyCrateAnnouncementcommand') || false;
    if (ToggleCrazyCrateAnnouncementcommand === 'false' || ToggleCrazyCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Crazy Crate!✨`);
    }
    CrazyallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getCrazySettingsDb = () => {
    const savedSettings = {
        CrazyCrateTimer: CrazySettingsDb.get('CrazyKittysCrateSettings:CrazyCrateTimer') || '2',
        CrazyCrate_Command: CrazySettingsDb.get('CrazyKittysCrateSettings:CrazyCrate_Command') || '!crazycrate',
        ToggleCrazyCrateChatcommand: CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyCrateChatcommand') || false,
        ToggleCrazyCrateAnnouncementcommand: CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyCrateAnnouncementcommand') || false,
        ToggleCrazyCoolDownSystemcommand: CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyCoolDownSystemcommand') || false,
        ToggleCrazyOpenAnimationcommand: CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazyOpenAnimationcommand') || false,
        ToggleCrazySoundcommand: CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazySoundcommand') || false,
        ToggleCrazySoundFilecommand: CrazySettingsDb.get('CrazyKittysCrateSettings:ToggleCrazySoundFilecommand') || '4',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function CrazyAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Crazy Crate')
        .body('Admin !Crazy Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await CrazyCrate_Form(sender);
        } else if (result.selection === 2) {
            await CrazyAdmin_Crate_Settings_Main(sender);
        } else {
            await CrazycloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function CrazyAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Crazy Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove / Edit Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const CrazysavedSettings = getCrazySettingsDb();
                const CrazyCrateTimerValue = Number(CrazysavedSettings.CrazyCrateTimer);
                const ToggleCrazyCrateChatcommandValue = CrazysavedSettings.ToggleCrazyCrateChatcommand === 'true' ? true : false;
                const ToggleCrazyCrateAnnouncementcommandValue = CrazysavedSettings.ToggleCrazyCrateAnnouncementcommand === 'true' ? true : false;
                const ToggleCrazyCoolDownSystemValue = CrazysavedSettings.ToggleCrazyCoolDownSystemcommand === 'true' ? true : false;
                const ToggleCrazyOpenAnimationValue = CrazysavedSettings.ToggleCrazyOpenAnimationcommand === 'true' ? true : false;
                const ToggleCrazySoundValue = CrazysavedSettings.ToggleCrazySoundcommand === 'true' ? true : false;
                const ToggleCrazySoundFileValue = Number(CrazysavedSettings.ToggleCrazySoundFilecommand)

                const form = new ModalFormData()
                    .title('Kittys Crate Crazy Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 48, 1, CrazyCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!crazycrate', CrazysavedSettings.CrazyCrate_Command)
                    .toggle(`Disable ${CrazysavedSettings.CrazyCrate_Command} Chat Command`, ToggleCrazyCrateChatcommandValue)
                    .toggle(`Disable !crazycrate Chat Prize Announcement`, ToggleCrazyCrateAnnouncementcommandValue)
                    .toggle(`Disable Crazy Crate Cool Down System`, ToggleCrazyCoolDownSystemValue)
                    .toggle(`Disable Crazy Crate Open Animation`, ToggleCrazyOpenAnimationValue)
                    .toggle(`Disable Crazy Crate Sound`, ToggleCrazySoundValue)
                    .slider(`Pick Crazy Crate Prize Sound✨`, 1, 12, 1, ToggleCrazySoundFileValue)


                form.show(player).then(response => {
                    const [CrazyCrateTimer, CrazyCrate_Command, ToggleCrazyCrateChatcommand, ToggleCrazyCrateAnnouncementcommand, ToggleCrazyCoolDownSystemcommand, ToggleCrazyOpenAnimationcommand, ToggleCrazySoundcommand, ToggleCrazySoundFilecommand] = response.formValues;
                    if (!CrazyCrate_Command || typeof CrazyCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return CrazyAdmin_Main_Menu(player);
                    }
                    CrazySettingsDb.set('CrazyKittysCrateSettings:ToggleCrazyCrateChatcommand', `${ToggleCrazyCrateChatcommand}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:ToggleCrazyCrateAnnouncementcommand', `${ToggleCrazyCrateAnnouncementcommand}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:ToggleCrazyCoolDownSystemcommand', `${ToggleCrazyCoolDownSystemcommand}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:ToggleCrazyOpenAnimationcommand', `${ToggleCrazyOpenAnimationcommand}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:ToggleCrazySoundcommand', `${ToggleCrazySoundcommand}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:ToggleCrazySoundFilecommand', `${ToggleCrazySoundFilecommand}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:CrazyCrateTimer', `${CrazyCrateTimer}`);
                    CrazySettingsDb.set('CrazyKittysCrateSettings:CrazyCrate_Command', CrazyCrate_Command);
                    return CrazyAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Crazy Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        CrazyItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return CrazyAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allCrazyItemsDb = CrazyItemsDb.getAll();
                    if (allCrazyItemsDb.length === 0) {
                        world.sendMessage("No Items to edit or remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allCrazyItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Crazy Items Menu")
                            .body(`Select an Item to Edit or Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allCrazyItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(CrazyItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity;

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` ${itemKey} (x${itemQuantity}) \n (Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(`§o§1Last Page\n§3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allCrazyItemsDb.length) {
                            form.button(`§o§1Next Page\n§3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allCrazyItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allCrazyItemsDb.length ? 1 : 0;

                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return CrazyAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allCrazyItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }

                        const itemToEdit = allCrazyItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(CrazyItemsDb.get(itemToEdit));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;

                        try {
                            let texturePath;
                            if (isitem.has(itemToEdit)) {
                                texturePath = `textures/items/${itemToEdit}`;
                            } else if (isblock.has(itemToEdit)) {
                                texturePath = `textures/blocks/${itemToEdit}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            const EditForm = new ActionFormData()
                                .title('Crazy Edit Item Menu')
                                .body(`Edit the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`)
                                .button("Edit Quantity✨", "textures/ui/icon_armor")
                                .button("Edit Rarity✨", "textures/ui/icon_armor")
                                .button("Edit ID✨", "textures/ui/icon_armor")
                                .button(`Remove Item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const editResponse = await EditForm.show(player);

                            if (editResponse.selection === 0) {
                                // Edit Item Quantity
                                const QuantityForm = new ModalFormData()
                                    .title("Edit Item Quantity")
                                    .slider(`Set new quantity for ${itemToEdit}✨`, 1, 64, 1, itemQuantity);
                                const quantityResponse = await QuantityForm.show(player);
                                const newQuantity = quantityResponse.formValues[0];

                                itemData.itemValue = newQuantity;
                                CrazyItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Quantity updated for ${itemToEdit}: now x${newQuantity}✨`);

                            } else if (editResponse.selection === 1) {
                                // Edit Item Rarity
                                const RarityForm = new ModalFormData()
                                    .title("Edit Item Rarity")
                                    .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, itemData.rarity);
                                const rarityResponse = await RarityForm.show(player);
                                const newRarity = rarityResponse.formValues[0];

                                itemData.rarity = newRarity;
                                CrazyItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Rarity updated for ${itemToEdit}: now ${newRarity}✨`);

                            } else if (editResponse.selection === 2) {
                                // Edit ID
                                const IdForm = new ModalFormData()
                                    .title("Edit Item ID")
                                    .textField(`Set new ID for ${itemToEdit}:`, "Enter new ID", `${itemToEdit}✨`);
                                const idResponse = await IdForm.show(player);
                                const newItemId = idResponse.formValues[0];

                                if (newItemId) {
                                    CrazyItemsDb.del(itemToEdit); // Remove the old ID entry
                                    CrazyItemsDb.set(newItemId, JSON.stringify(itemData)); // Save with new ID
                                    world.sendMessage(`Item ID updated: ${itemToEdit} → ${newItemId}✨`);
                                    return
                                } else {
                                    world.sendMessage("Invalid item ID.✨");
                                }
                            } else if (editResponse.selection === 3) {
                                // Confirm Removal
                                const ConfirmForm = new ActionFormData()
                                    .title('Crazy Confirm Item Removal')
                                    .body(`Are you sure you want to remove the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                    .button(`Remove: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`, texturePath)
                                    .button("Go Back✨", "textures/ui/icon_map");

                                const confirmResponse = await ConfirmForm.show(player);

                                if (confirmResponse.selection === 0) {
                                    CrazyItemsDb.del(itemToEdit);
                                    world.sendMessage(`Item removed: ${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the edit menu.✨", error);
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function CrazycloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:crazychest") return
    if (player && data.itemStack && data.itemStack.typeId === "kittys:removal_tool") {
        return;
    };
    const CrazycrateTimerValue = CrazySettingsDb.get('KittysCrazyCrateSettings:CrazyCrateTimer') || '2';
    const CrazycrateTimerHours = Number(CrazycrateTimerValue);
    const cooldownTime = CrazycrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await CrazyAdmin_Main_Menu(data.player);
        return;
    }
    if (player && data.itemStack && data.itemStack.typeId === "kittys:crazycrate_key") {
        await data.player.sendMessage('§6§lOpening Crate Menu!✨');
        system.run(() => {
            target.dimension.runCommand(`clear "${player.name}" kittys:crazycrate_key 0 1`)
            target.dimension.runCommand(`say §c§lCrazy Key Used!✨`);

            target.dimension.runCommand(`tag "${player.name}" add Crazykeyed`)
        });

        const CrazycrateMenu = new ActionFormData()
            .title('!Crazy Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.player, CrazycrateMenu);
            result.selection === 0 ? await CrazyCrate_Form(data.player) : await CrazycloseCrateMenu(data.player);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Crazycooldowns.has(player.name)) return false;
        const cooldown = Crazycooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    const CrazysavedSettings = getCrazySettingsDb();
    const ToggleCrazyCoolDownSystemValue = CrazysavedSettings.ToggleCrazyCoolDownSystemcommand === 'true';
    if (ToggleCrazyCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Crazycooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the crazy crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');

    const CrazycrateMenu = new ActionFormData()
        .title('!Crazy Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, CrazycrateMenu);
        result.selection === 0 ? await CrazyCrate_Form(data.player) : await CrazycloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});















// ██████╗  ██████╗ ██╗    ██╗███████╗██████╗      ██████╗██████╗  █████╗ ████████╗███████╗        
// ██╔══██╗██╔═══██╗██║    ██║██╔════╝██╔══██╗    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝        
// ██████╔╝██║   ██║██║ █╗ ██║█████╗  ██████╔╝    ██║     ██████╔╝███████║   ██║   █████╗          
// ██╔═══╝ ██║   ██║██║███╗██║██╔══╝  ██╔══██╗    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝          
// ██║     ╚██████╔╝╚███╔███╔╝███████╗██║  ██║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗        
// ╚═╝      ╚═════╝  ╚══╝╚══╝ ╚══════╝╚═╝  ╚═╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝                                                                                                                                                                                                            
//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const TogglePowerCrateChatcommand = PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerCrateChatcommand') || false;
    if (TogglePowerCrateChatcommand === 'true' || TogglePowerCrateChatcommand === true) {
        return;
    }
    const PowerCrate_Command = PowerSettingsDb.get('PowerKittysCrateSettings:PowerCrate_Command') || '!powercrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${PowerCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = PowerSettingsDb.get('PowerKittysCrateSettings:PowerCrate_Command') || '!powercrate';
    const crateTimerValue = PowerSettingsDb.get('PowerKittysCrateSettings:PowerCrateTimer') || '2';
    const TogglePowerCrateChatcommand = PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerCrateChatcommand') || false;
    if (TogglePowerCrateChatcommand === 'true' || TogglePowerCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await PowerAdmin_Main_Menu(data.sender);
        return;
    }

    async function checkItemAmount(player) {
        const component = player.getComponent("minecraft:inventory");
        const inventory = component.container;
        let itemAmount = 0;
        for (let i = 0; i < 36; i++) {
            let item = inventory.getItem(i);
            if (item?.typeId === "kittys:powercrate_key") {
                itemAmount += item.amount;
            }
        }
        return itemAmount;
    }

    const itemAmount = await checkItemAmount(data.sender);

    if (itemAmount >= 1) {
        system.run(() => {
            data.sender.runCommand(`clear "${data.sender.name}" kittys:powercrate_key 0 1`);
            data.sender.runCommand(`tag "${data.sender.name}" add Powerkeyed`);
            data.sender.sendMessage(`§c§lPower Key Used!✨`);
        });

        await data.sender.sendMessage('§6§lOpening Crate Menu!✨');
        const PowercrateMenu = new ActionFormData()
            .title('!Power Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.sender, PowercrateMenu);
            result.selection === 0 ? await PowerCrate_Form(data.sender) : await PowercloseCrateMenu(data.sender);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }

    const PowersavedSettings = getPowerSettingsDb();
    const TogglePowerCoolDownSystemValue = PowersavedSettings.TogglePowerCoolDownSystemcommand === 'true';
    if (TogglePowerCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Powercooldowns.has(player.name)) return false;

        const playerCooldown = Powercooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Powercooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the power crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Power Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await PowerCrate_Form(data.sender) : await PowercloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function PowerCrate_Form(player) {
    const crateTimerValue = PowerSettingsDb.get('PowerKittysCrateSettings:PowerCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Powercooldowns.has(player.name)) return false;
        const cooldown = Powercooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner") ||
        player.hasTag("Powerkeyed")
    ) {
        const allPrizes = getPowerPrizes();
        const randomPrize = getPower_RandomPrize(allPrizes);
        await givePlayerPower_Prize(player, randomPrize);
        await Power_Crate_Result_Form(player, randomPrize, allPrizes);
        const overworld = world.getDimension('overworld');
        await overworld.runCommand(`tag "${player.name}" remove Powerkeyed`)
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Powercooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the power crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getPowerPrizes();
    const randomPrize = getPower_RandomPrize(allPrizes);
    await givePlayerPower_Prize(player, randomPrize);
    await Power_Crate_Result_Form(player, randomPrize, allPrizes);

    Powercooldowns.set(player.name, Date.now());
}

const getPowerPrizes = () => {
    const customPrizes = PowerItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Powerprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(PowerItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getPower_RandomPrize(Powerprizes) {
    const totalWeight = Powerprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Powerprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerPower_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    await overworld.runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
    await overworld.runCommandAsync(`scoreboard players add "${player.name}" cratesopened 1`);
    const PowersavedSettings = getPowerSettingsDb();
    const TogglePowerOpenAnimationSystemValue = PowersavedSettings.TogglePowerOpenAnimationcommand === 'true';
    if (TogglePowerOpenAnimationSystemValue === false) {
        await overworld.runCommand("function powerchest")
    }
    if (!player.hasTag("Powerkeyed")
    ) {
        PowersetCooldown(player.name);
    }
    const TogglePowerSoundSystemValue = PowersavedSettings.TogglePowerSoundcommand === 'true';
    if (TogglePowerSoundSystemValue === false) {
        const TogglePowerSoundFileValue = Number(PowersavedSettings.TogglePowerSoundFilecommand);
        playCrateSound(TogglePowerSoundFileValue, player);
    }
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Power_Crate_Result_Form(player, wonPrize, PowerallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Power Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const TogglePowerCrateAnnouncementcommand = PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerCrateAnnouncementcommand') || false;
    if (TogglePowerCrateAnnouncementcommand === 'false' || TogglePowerCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Power Crate!✨`);
    }
    PowerallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getPowerSettingsDb = () => {
    const savedSettings = {
        PowerCrateTimer: PowerSettingsDb.get('PowerKittysCrateSettings:PowerCrateTimer') || '2',
        PowerCrate_Command: PowerSettingsDb.get('PowerKittysCrateSettings:PowerCrate_Command') || '!powercrate',
        TogglePowerCrateChatcommand: PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerCrateChatcommand') || false,
        TogglePowerCrateAnnouncementcommand: PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerCrateAnnouncementcommand') || false,
        TogglePowerCoolDownSystemcommand: PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerCoolDownSystemcommand') || false,
        TogglePowerOpenAnimationcommand: PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerOpenAnimationcommand') || false,
        TogglePowerSoundcommand: PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerSoundcommand') || false,
        TogglePowerSoundFilecommand: PowerSettingsDb.get('PowerKittysCrateSettings:TogglePowerSoundFilecommand') || '5',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function PowerAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Power Crate')
        .body('Admin !Power Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await PowerCrate_Form(sender);
        } else if (result.selection === 2) {
            await PowerAdmin_Crate_Settings_Main(sender);
        } else {
            await PowercloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function PowerAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Power Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove / Edit Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const PowersavedSettings = getPowerSettingsDb();
                const PowerCrateTimerValue = Number(PowersavedSettings.PowerCrateTimer);
                const TogglePowerCrateChatcommandValue = PowersavedSettings.TogglePowerCrateChatcommand === 'true' ? true : false;
                const TogglePowerCrateAnnouncementcommandValue = PowersavedSettings.TogglePowerCrateAnnouncementcommand === 'true' ? true : false;
                const TogglePowerCoolDownSystemValue = PowersavedSettings.TogglePowerCoolDownSystemcommand === 'true' ? true : false;
                const TogglePowerOpenAnimationValue = PowersavedSettings.TogglePowerOpenAnimationcommand === 'true' ? true : false;
                const TogglePowerSoundValue = PowersavedSettings.TogglePowerSoundcommand === 'true' ? true : false;
                const TogglePowerSoundFileValue = Number(PowersavedSettings.TogglePowerSoundFilecommand)

                const form = new ModalFormData()
                    .title('Kittys Crate Power Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 48, 1, PowerCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!powercrate', PowersavedSettings.PowerCrate_Command)
                    .toggle(`Disable ${PowersavedSettings.PowerCrate_Command} Chat Command`, TogglePowerCrateChatcommandValue)
                    .toggle(`Disable !powercrate Chat Prize Announcement`, TogglePowerCrateAnnouncementcommandValue)
                    .toggle(`Disable Power Crate Cool Down System`, TogglePowerCoolDownSystemValue)
                    .toggle(`Disable Power Crate Open Animation`, TogglePowerOpenAnimationValue)
                    .toggle(`Disable Power Crate Sound`, TogglePowerSoundValue)
                    .slider(`Pick Power Crate Prize Sound✨`, 1, 12, 1, TogglePowerSoundFileValue)


                form.show(player).then(response => {
                    const [PowerCrateTimer, PowerCrate_Command, TogglePowerCrateChatcommand, TogglePowerCrateAnnouncementcommand, TogglePowerCoolDownSystemcommand, TogglePowerOpenAnimationcommand, TogglePowerSoundcommand, TogglePowerSoundFilecommand] = response.formValues;
                    if (!PowerCrate_Command || typeof PowerCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return PowerAdmin_Main_Menu(player);
                    }
                    PowerSettingsDb.set('PowerKittysCrateSettings:TogglePowerCrateChatcommand', `${TogglePowerCrateChatcommand}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:TogglePowerCrateAnnouncementcommand', `${TogglePowerCrateAnnouncementcommand}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:TogglePowerCoolDownSystemcommand', `${TogglePowerCoolDownSystemcommand}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:TogglePowerOpenAnimationcommand', `${TogglePowerOpenAnimationcommand}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:TogglePowerSoundcommand', `${TogglePowerSoundcommand}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:TogglePowerSoundFilecommand', `${TogglePowerSoundFilecommand}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:PowerCrateTimer', `${PowerCrateTimer}`);
                    PowerSettingsDb.set('PowerKittysCrateSettings:PowerCrate_Command', PowerCrate_Command);
                    return PowerAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Power Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        PowerItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return PowerAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allPowerItemsDb = PowerItemsDb.getAll();
                    if (allPowerItemsDb.length === 0) {
                        world.sendMessage("No Items to edit or remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allPowerItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Power Items Menu")
                            .body(`Select an Item to Edit or Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allPowerItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(PowerItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity;

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` ${itemKey} (x${itemQuantity}) \n (Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(`§o§1Last Page\n§3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allPowerItemsDb.length) {
                            form.button(`§o§1Next Page\n§3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allPowerItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allPowerItemsDb.length ? 1 : 0;

                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return PowerAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allPowerItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }

                        const itemToEdit = allPowerItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(PowerItemsDb.get(itemToEdit));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;

                        try {
                            let texturePath;
                            if (isitem.has(itemToEdit)) {
                                texturePath = `textures/items/${itemToEdit}`;
                            } else if (isblock.has(itemToEdit)) {
                                texturePath = `textures/blocks/${itemToEdit}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            const EditForm = new ActionFormData()
                                .title('Power Edit Item Menu')
                                .body(`Edit the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`)
                                .button("Edit Quantity✨", "textures/ui/icon_armor")
                                .button("Edit Rarity✨", "textures/ui/icon_armor")
                                .button("Edit ID✨", "textures/ui/icon_armor")
                                .button(`Remove Item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const editResponse = await EditForm.show(player);

                            if (editResponse.selection === 0) {
                                // Edit Item Quantity
                                const QuantityForm = new ModalFormData()
                                    .title("Edit Item Quantity")
                                    .slider(`Set new quantity for ${itemToEdit}✨`, 1, 64, 1, itemQuantity);
                                const quantityResponse = await QuantityForm.show(player);
                                const newQuantity = quantityResponse.formValues[0];

                                itemData.itemValue = newQuantity;
                                PowerItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Quantity updated for ${itemToEdit}: now x${newQuantity}✨`);

                            } else if (editResponse.selection === 1) {
                                // Edit Item Rarity
                                const RarityForm = new ModalFormData()
                                    .title("Edit Item Rarity")
                                    .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, itemData.rarity);
                                const rarityResponse = await RarityForm.show(player);
                                const newRarity = rarityResponse.formValues[0];

                                itemData.rarity = newRarity;
                                PowerItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Rarity updated for ${itemToEdit}: now ${newRarity}✨`);

                            } else if (editResponse.selection === 2) {
                                // Edit ID
                                const IdForm = new ModalFormData()
                                    .title("Edit Item ID")
                                    .textField(`Set new ID for ${itemToEdit}:`, "Enter new ID", `${itemToEdit}✨`);
                                const idResponse = await IdForm.show(player);
                                const newItemId = idResponse.formValues[0];

                                if (newItemId) {
                                    PowerItemsDb.del(itemToEdit); // Remove the old ID entry
                                    PowerItemsDb.set(newItemId, JSON.stringify(itemData)); // Save with new ID
                                    world.sendMessage(`Item ID updated: ${itemToEdit} → ${newItemId}✨`);
                                    return
                                } else {
                                    world.sendMessage("Invalid item ID.✨");
                                }
                            } else if (editResponse.selection === 3) {
                                // Confirm Removal
                                const ConfirmForm = new ActionFormData()
                                    .title('Power Confirm Item Removal')
                                    .body(`Are you sure you want to remove the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                    .button(`Remove: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`, texturePath)
                                    .button("Go Back✨", "textures/ui/icon_map");

                                const confirmResponse = await ConfirmForm.show(player);

                                if (confirmResponse.selection === 0) {
                                    PowerItemsDb.del(itemToEdit);
                                    world.sendMessage(`Item removed: ${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the edit menu.✨", error);
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function PowercloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:powerchest") return
    if (player && data.itemStack && data.itemStack.typeId === "kittys:removal_tool") {
        return;
    };
    const PowercrateTimerValue = PowerSettingsDb.get('KittysPowerCrateSettings:PowerCrateTimer') || '2';
    const PowercrateTimerHours = Number(PowercrateTimerValue);
    const cooldownTime = PowercrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await PowerAdmin_Main_Menu(data.player);
        return;
    }
    if (player && data.itemStack && data.itemStack.typeId === "kittys:powercrate_key") {
        await data.player.sendMessage('§6§lOpening Crate Menu!✨');
        system.run(() => {
            target.dimension.runCommand(`clear "${player.name}" kittys:powercrate_key 0 1`)
            target.dimension.runCommand(`say §c§lPower Key Used!✨`);

            target.dimension.runCommand(`tag "${player.name}" add Powerkeyed`)
        });

        const PowercrateMenu = new ActionFormData()
            .title('!Power Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.player, PowercrateMenu);
            result.selection === 0 ? await PowerCrate_Form(data.player) : await PowercloseCrateMenu(data.player);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Powercooldowns.has(player.name)) return false;
        const cooldown = Powercooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    const PowersavedSettings = getPowerSettingsDb();
    const TogglePowerCoolDownSystemValue = PowersavedSettings.TogglePowerCoolDownSystemcommand === 'true';
    if (TogglePowerCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Powercooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the power crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');

    const PowercrateMenu = new ActionFormData()
        .title('!Power Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, PowercrateMenu);
        result.selection === 0 ? await PowerCrate_Form(data.player) : await PowercloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});














// ███╗   ███╗██╗   ██╗███████╗████████╗██╗ ██████╗     ██████╗██████╗  █████╗ ████████╗███████╗        
// ████╗ ████║╚██╗ ██╔╝██╔════╝╚══██╔══╝██║██╔════╝    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝        
// ██╔████╔██║ ╚████╔╝ ███████╗   ██║   ██║██║         ██║     ██████╔╝███████║   ██║   █████╗          
// ██║╚██╔╝██║  ╚██╔╝  ╚════██║   ██║   ██║██║         ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝          
// ██║ ╚═╝ ██║   ██║   ███████║   ██║   ██║╚██████╗    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗        
// ╚═╝     ╚═╝   ╚═╝   ╚══════╝   ╚═╝   ╚═╝ ╚═════╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝                                                                                                                                                                                                                     
//  ██████╗██╗  ██╗ █████╗ ████████╗     ██████╗ ██████╗ ███╗   ███╗███╗   ███╗ █████╗ ███╗   ██╗██████╗ 
// ██╔════╝██║  ██║██╔══██╗╚══██╔══╝    ██╔════╝██╔═══██╗████╗ ████║████╗ ████║██╔══██╗████╗  ██║██╔══██╗
// ██║     ███████║███████║   ██║       ██║     ██║   ██║██╔████╔██║██╔████╔██║███████║██╔██╗ ██║██║  ██║
// ██║     ██╔══██║██╔══██║   ██║       ██║     ██║   ██║██║╚██╔╝██║██║╚██╔╝██║██╔══██║██║╚██╗██║██║  ██║
// ╚██████╗██║  ██║██║  ██║   ██║       ╚██████╗╚██████╔╝██║ ╚═╝ ██║██║ ╚═╝ ██║██║  ██║██║ ╚████║██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝        ╚═════╝ ╚═════╝ ╚═╝     ╚═╝╚═╝     ╚═╝╚═╝  ╚═╝╚═╝  ╚═══╝╚═════╝ 
system.runInterval(() => {
    const ToggleMysticCrateChatcommand = MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticCrateChatcommand') || false;
    if (ToggleMysticCrateChatcommand === 'true' || ToggleMysticCrateChatcommand === true) {
        return;
    }
    const MysticCrate_Command = MysticSettingsDb.get('MysticKittysCrateSettings:MysticCrate_Command') || '!mysticcrate';
    world.getDimension('overworld').runCommand(`say Open the crate menu with §6${MysticCrate_Command}`);
}, 15000);

world.beforeEvents.chatSend.subscribe(async (data) => {
    const Crate_Command = MysticSettingsDb.get('MysticKittysCrateSettings:MysticCrate_Command') || '!mysticcrate';
    const crateTimerValue = MysticSettingsDb.get('MysticKittysCrateSettings:MysticCrateTimer') || '2';
    const ToggleMysticCrateChatcommand = MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticCrateChatcommand') || false;
    if (ToggleMysticCrateChatcommand === 'true' || ToggleMysticCrateChatcommand === true) {
        return;
    }
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    if (!data.message.startsWith(Crate_Command)) return;
    data.cancel = true;

    if (
        data.sender.hasTag("Admin") ||
        data.sender.hasTag("admin") ||
        data.sender.hasTag("Staff") ||
        data.sender.hasTag("staff") ||
        data.sender.hasTag("Owner") ||
        data.sender.hasTag("owner")
    ) {
        await MysticAdmin_Main_Menu(data.sender);
        return;
    }

    async function checkItemAmount(player) {
        const component = player.getComponent("minecraft:inventory");
        const inventory = component.container;
        let itemAmount = 0;
        for (let i = 0; i < 36; i++) {
            let item = inventory.getItem(i);
            if (item?.typeId === "kittys:mysticcrate_key") {
                itemAmount += item.amount;
            }
        }
        return itemAmount;
    }

    const itemAmount = await checkItemAmount(data.sender);

    if (itemAmount >= 1) {
        system.run(() => {
            data.sender.runCommand(`clear "${data.sender.name}" kittys:mysticcrate_key 0 1`);
            data.sender.runCommand(`tag "${data.sender.name}" add Mystickeyed`);
            data.sender.sendMessage(`§c§lMystic Key Used!✨`);
        });

        await data.sender.sendMessage('§6§lOpening Crate Menu!✨');
        const MysticcrateMenu = new ActionFormData()
            .title('!Mystic Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.sender, MysticcrateMenu);
            result.selection === 0 ? await MysticCrate_Form(data.sender) : await MysticcloseCrateMenu(data.sender);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }

    const MysticsavedSettings = getMysticSettingsDb();
    const ToggleMysticCoolDownSystemValue = MysticsavedSettings.ToggleMysticCoolDownSystemcommand === 'true';
    if (ToggleMysticCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Mysticcooldowns.has(player.name)) return false;

        const playerCooldown = Mysticcooldowns.get(player.name);
        return (now - playerCooldown) < cooldownTime;
    }

    if (isOnCooldown(data.sender)) {
        const remainingTime = cooldownTime - (Date.now() - Mysticcooldowns.get(data.sender.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.sender.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the mystic crate again.✨`);
        data.sender.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await data.sender.playSound('note.pling', { volume: 1 });


    const crateMenu = new ActionFormData()
        .title('!Mystic Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.sender, crateMenu);
        result.selection === 0 ? await MysticCrate_Form(data.sender) : await MysticcloseCrateMenu(data.sender);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

//  ██████╗██████╗  █████╗ ████████╗███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ██║     ██████╔╝███████║   ██║   █████╗      ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                   
async function MysticCrate_Form(player) {
    const crateTimerValue = MysticSettingsDb.get('MysticKittysCrateSettings:MysticCrateTimer') || '2';
    const crateTimerHours = Number(crateTimerValue);
    const cooldownTime = crateTimerHours * 60 * 60 * 1000; // hours to milliseconds

    function isOnCooldown(player) {
        const now = Date.now();
        if (!Mysticcooldowns.has(player.name)) return false;
        const cooldown = Mysticcooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    if (
        player.hasTag("Admin") ||
        player.hasTag("admin") ||
        player.hasTag("Staff") ||
        player.hasTag("staff") ||
        player.hasTag("Owner") ||
        player.hasTag("owner") ||
        player.hasTag("Mystickeyed")
    ) {
        const allPrizes = getMysticPrizes();
        const randomPrize = getMystic_RandomPrize(allPrizes);
        await givePlayerMystic_Prize(player, randomPrize);
        await Mystic_Crate_Result_Form(player, randomPrize, allPrizes);
        const overworld = world.getDimension('overworld');
        await overworld.runCommand(`tag "${player.name}" remove Mystickeyed`)
        return;
    }
    if (isOnCooldown(player)) {
        const remainingTime = cooldownTime - (Date.now() - Mysticcooldowns.get(player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the mystic crate again.✨`);
        player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    const allPrizes = getMysticPrizes();
    const randomPrize = getMystic_RandomPrize(allPrizes);
    await givePlayerMystic_Prize(player, randomPrize);
    await Mystic_Crate_Result_Form(player, randomPrize, allPrizes);

    Mysticcooldowns.set(player.name, Date.now());
}

const getMysticPrizes = () => {
    const customPrizes = MysticItemsDb.getAll();
    if (customPrizes.length === 0) {
        return Mysticprizes;
    }
    return customPrizes.map(itemKey => {
        const itemData = JSON.parse(MysticItemsDb.get(itemKey));
        return {
            item: itemKey,
            quantity: itemData.itemValue || 1,
            weight: itemData.weight || 1,
        };
    });
};

function getMystic_RandomPrize(Mysticprizes) {
    const totalWeight = Mysticprizes.reduce((sum, prize) => sum + prize.weight, 0);
    const randomNum = Math.floor(Math.random() * totalWeight);

    let runningSum = 0;
    for (const prize of Mysticprizes) {
        runningSum += prize.weight;
        if (randomNum < runningSum) return prize;
    }
}

async function givePlayerMystic_Prize(player, prize) {
    const overworld = world.getDimension('overworld');
    await overworld.runCommandAsync(`give "${player.name}" ${prize.item} ${prize.quantity}`);
    await overworld.runCommandAsync(`scoreboard objectives add "cratesopened" dummy`);
    await overworld.runCommandAsync(`scoreboard players add "${player.name}" cratesopened 1`);
    const MysticsavedSettings = getMysticSettingsDb();
    const ToggleMysticOpenAnimationSystemValue = MysticsavedSettings.ToggleMysticOpenAnimationcommand === 'true';
    if (ToggleMysticOpenAnimationSystemValue === false) {
        await overworld.runCommand("function mysticchest")
    }
    if (!player.hasTag("Mystickeyed")
    ) {
        MysticsetCooldown(player.name);
    }
    const ToggleMysticSoundSystemValue = MysticsavedSettings.ToggleMysticSoundcommand === 'true';
    if (ToggleMysticSoundSystemValue === false) {
        const ToggleMysticSoundFileValue = Number(MysticsavedSettings.ToggleMysticSoundFilecommand);
        playCrateSound(ToggleMysticSoundFileValue, player);
    }
}

//  ██████╗██████╗  █████╗ ████████╗███████╗    ██████╗ ███████╗███████╗██╗   ██╗██╗  ████████╗    ███████╗ ██████╗ ██████╗ ███╗   ███╗
// ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔══██╗██╔════╝██╔════╝██║   ██║██║  ╚══██╔══╝    ██╔════╝██╔═══██╗██╔══██╗████╗ ████║
// ██║     ██████╔╝███████║   ██║   █████╗      ██████╔╝█████╗  ███████╗██║   ██║██║     ██║       █████╗  ██║   ██║██████╔╝██╔████╔██║
// ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ██╔══██╗██╔══╝  ╚════██║██║   ██║██║     ██║       ██╔══╝  ██║   ██║██╔══██╗██║╚██╔╝██║
// ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ██║  ██║███████╗███████║╚██████╔╝███████╗██║       ██║     ╚██████╔╝██║  ██║██║ ╚═╝ ██║
//  ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚═╝  ╚═╝╚══════╝╚══════╝ ╚═════╝ ╚══════╝╚═╝       ╚═╝      ╚═════╝ ╚═╝  ╚═╝╚═╝     ╚═╝                                                                                                                                                                                                                                                             
async function Mystic_Crate_Result_Form(player, wonPrize, MysticallPrizes) {
    const spinResultForm = new ActionFormData()
        .title(' !Mystic Crate Opened!')

    let texturePath;
    if (isitem.has(wonPrize.item)) {
        texturePath = `textures/items/${wonPrize.item}`;
    } else if (isblock.has(wonPrize.item)) {
        texturePath = `textures/blocks/${wonPrize.item}`;
    } else {
        texturePath = `textures/ui/gift_square`;
    }
    spinResultForm.button(`You Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''}!✨`, texturePath);

    const ToggleMysticCrateAnnouncementcommand = MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticCrateAnnouncementcommand') || false;
    if (ToggleMysticCrateAnnouncementcommand === 'false' || ToggleMysticCrateAnnouncementcommand === false) {
        world.sendMessage(`${player.name} Won: ${wonPrize.quantity} ${wonPrize.item.replace(/_/g, ' ')}${wonPrize.quantity > 1 ? 's' : ''} From The Mystic Crate!✨`);
    }
    MysticallPrizes
        .filter(prize => prize !== wonPrize)
        .forEach(prize => {
            let texturePath;
            if (isitem.has(prize.item)) {
                texturePath = `textures/items/${prize.item}`;
            } else if (isblock.has(prize.item)) {
                texturePath = `textures/blocks/${prize.item}`;
            } else {
                texturePath = `textures/ui/gift_square`;
            }

            spinResultForm.button(`${prize.quantity} ${prize.item.replace(/_/g, ' ')}${prize.quantity > 1 ? 's' : ''}✨`, texturePath);
        });

    spinResultForm.button('Close');

    await spinResultForm.show(player);
    try {
        await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
        await player.playSound('note.bass', { volume: 0.5 });
    } catch (error) {
        console.error(`Error displaying spin result: ${error.message}`);
    }
}

const getMysticSettingsDb = () => {
    const savedSettings = {
        MysticCrateTimer: MysticSettingsDb.get('MysticKittysCrateSettings:MysticCrateTimer') || '2',
        MysticCrate_Command: MysticSettingsDb.get('MysticKittysCrateSettings:MysticCrate_Command') || '!mysticcrate',
        ToggleMysticCrateChatcommand: MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticCrateChatcommand') || false,
        ToggleMysticCrateAnnouncementcommand: MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticCrateAnnouncementcommand') || false,
        ToggleMysticCoolDownSystemcommand: MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticCoolDownSystemcommand') || false,
        ToggleMysticOpenAnimationcommand: MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticOpenAnimationcommand') || false,
        ToggleMysticSoundcommand: MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticSoundcommand') || false,
        ToggleMysticSoundFilecommand: MysticSettingsDb.get('MysticKittysCrateSettings:ToggleMysticSoundFilecommand') || '6',
    };
    return savedSettings;
};

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ████╗ ████║██╔══██╗██║████╗  ██║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██╔████╔██║███████║██║██╔██╗ ██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝ 
async function MysticAdmin_Main_Menu(sender) {
    await sender.sendMessage('§6§lPlease close the chat to access the Crate Menu!✨');
    await sender.playSound('note.pling', { volume: 1 });

    const crateMenu = new ActionFormData()
        .title('Admin !Mystic Crate')
        .body('Admin !Mystic Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel')
        .button('Settings', 'textures/ui/gear');

    try {
        const result = await forceShow(sender, crateMenu);
        if (result.selection === 0) {
            await MysticCrate_Form(sender);
        } else if (result.selection === 2) {
            await MysticAdmin_Crate_Settings_Main(sender);
        } else {
            await MysticcloseCrateMenu(sender);
        }
    } catch (error) {
        console.error(`Error displaying crate menu: ${error.message}`);
    }
}

//  █████╗ ██████╗ ███╗   ███╗██╗███╗   ██╗     ██████╗██████╗  █████╗ ████████╗███████╗    ███████╗███████╗████████╗████████╗██╗███╗   ██╗ ██████╗ ███████╗    ███╗   ███╗ █████╗ ██╗███╗   ██╗
// ██╔══██╗██╔══██╗████╗ ████║██║████╗  ██║    ██╔════╝██╔══██╗██╔══██╗╚══██╔══╝██╔════╝    ██╔════╝██╔════╝╚══██╔══╝╚══██╔══╝██║████╗  ██║██╔════╝ ██╔════╝    ████╗ ████║██╔══██╗██║████╗  ██║
// ███████║██║  ██║██╔████╔██║██║██╔██╗ ██║    ██║     ██████╔╝███████║   ██║   █████╗      ███████╗█████╗     ██║      ██║   ██║██╔██╗ ██║██║  ███╗███████╗    ██╔████╔██║███████║██║██╔██╗ ██║
// ██╔══██║██║  ██║██║╚██╔╝██║██║██║╚██╗██║    ██║     ██╔══██╗██╔══██║   ██║   ██╔══╝      ╚════██║██╔══╝     ██║      ██║   ██║██║╚██╗██║██║   ██║╚════██║    ██║╚██╔╝██║██╔══██║██║██║╚██╗██║
// ██║  ██║██████╔╝██║ ╚═╝ ██║██║██║ ╚████║    ╚██████╗██║  ██║██║  ██║   ██║   ███████╗    ███████║███████╗   ██║      ██║   ██║██║ ╚████║╚██████╔╝███████║    ██║ ╚═╝ ██║██║  ██║██║██║ ╚████║
// ╚═╝  ╚═╝╚═════╝ ╚═╝     ╚═╝╚═╝╚═╝  ╚═══╝     ╚═════╝╚═╝  ╚═╝╚═╝  ╚═╝   ╚═╝   ╚══════╝    ╚══════╝╚══════╝   ╚═╝      ╚═╝   ╚═╝╚═╝  ╚═══╝ ╚═════╝ ╚══════╝    ╚═╝     ╚═╝╚═╝  ╚═╝╚═╝╚═╝  ╚═══╝                                                                                                                                                                                      
async function MysticAdmin_Crate_Settings_Main(player) {
    const customUi = new ActionFormData()
        .title("Kittys Mystic Crate Settings Main Menu")
        .body("")
        .button("Crate Settings", "textures/ui/accessibility_glyph_color")
        .button("Add Item", "textures/ui/icon_best3")
        .button("Remove / Edit Item", "textures/ui/enable_editor");

    customUi.show(player).then(response => {

        switch (response.selection) {

            case 0:
                const MysticsavedSettings = getMysticSettingsDb();
                const MysticCrateTimerValue = Number(MysticsavedSettings.MysticCrateTimer);
                const ToggleMysticCrateChatcommandValue = MysticsavedSettings.ToggleMysticCrateChatcommand === 'true' ? true : false;
                const ToggleMysticCrateAnnouncementcommandValue = MysticsavedSettings.ToggleMysticCrateAnnouncementcommand === 'true' ? true : false;
                const ToggleMysticCoolDownSystemValue = MysticsavedSettings.ToggleMysticCoolDownSystemcommand === 'true' ? true : false;
                const ToggleMysticOpenAnimationValue = MysticsavedSettings.ToggleMysticOpenAnimationcommand === 'true' ? true : false;
                const ToggleMysticSoundValue = MysticsavedSettings.ToggleMysticSoundcommand === 'true' ? true : false;
                const ToggleMysticSoundFileValue = Number(MysticsavedSettings.ToggleMysticSoundFilecommand)

                const form = new ModalFormData()
                    .title('Kittys Crate Mystic Settings Menu')
                    .slider(`Crate Reset Time (Hours)✨`, 1, 48, 1, MysticCrateTimerValue)
                    .textField('Set The Crate Command,✨\nThis Will Be The New Command To Open The Crate Menu', '!mysticcrate', MysticsavedSettings.MysticCrate_Command)
                    .toggle(`Disable ${MysticsavedSettings.MysticCrate_Command} Chat Command`, ToggleMysticCrateChatcommandValue)
                    .toggle(`Disable !mysticcrate Chat Prize Announcement`, ToggleMysticCrateAnnouncementcommandValue)
                    .toggle(`Disable Mystic Crate Cool Down System`, ToggleMysticCoolDownSystemValue)
                    .toggle(`Disable Mystic Crate Open Animation`, ToggleMysticOpenAnimationValue)
                    .toggle(`Disable Mystic Crate Sound`, ToggleMysticSoundValue)
                    .slider(`Pick Mystic Crate Prize Sound✨`, 1, 12, 1, ToggleMysticSoundFileValue)


                form.show(player).then(response => {
                    const [MysticCrateTimer, MysticCrate_Command, ToggleMysticCrateChatcommand, ToggleMysticCrateAnnouncementcommand, ToggleMysticCoolDownSystemcommand, ToggleMysticOpenAnimationcommand, ToggleMysticSoundcommand, ToggleMysticSoundFilecommand] = response.formValues;
                    if (!MysticCrate_Command || typeof MysticCrate_Command !== 'string') {
                        world.sendMessage("⚠️ Please provide a valid crate command.");
                        player.playSound('note.bass', { volume: 0.5 });
                        return MysticAdmin_Main_Menu(player);
                    }
                    MysticSettingsDb.set('MysticKittysCrateSettings:ToggleMysticCrateChatcommand', `${ToggleMysticCrateChatcommand}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:ToggleMysticCrateAnnouncementcommand', `${ToggleMysticCrateAnnouncementcommand}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:ToggleMysticCoolDownSystemcommand', `${ToggleMysticCoolDownSystemcommand}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:ToggleMysticOpenAnimationcommand', `${ToggleMysticOpenAnimationcommand}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:ToggleMysticSoundcommand', `${ToggleMysticSoundcommand}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:ToggleMysticSoundFilecommand', `${ToggleMysticSoundFilecommand}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:MysticCrateTimer', `${MysticCrateTimer}`);
                    MysticSettingsDb.set('MysticKittysCrateSettings:MysticCrate_Command', MysticCrate_Command);
                    return MysticAdmin_Main_Menu(player);
                });
                break;

            //  █████╗ ██████╗ ██████╗     ██╗████████╗███████╗███╗   ███╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔══██╗██╔══██╗    ██║╚══██╔══╝██╔════╝████╗ ████║    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ███████║██║  ██║██║  ██║    ██║   ██║   █████╗  ██╔████╔██║    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██║██║  ██║██║  ██║    ██║   ██║   ██╔══╝  ██║╚██╔╝██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║██████╔╝██████╔╝    ██║   ██║   ███████╗██║ ╚═╝ ██║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚═════╝ ╚═════╝     ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                     
            case 1:
                (async () => {
                    const form = new ModalFormData()
                        .title('Kittys Mystic Add Item Menu')
                        .textField('Add Item\nThis Will Display On The Crate Prize Screen:✨', 'apple, iron_sword')
                        .slider(`Quantity✨`, 1, 64, 1, 1)
                        .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, 1);

                    const response = await form.show(player);
                    try {
                        const itemName = response.formValues[0];
                        if (!itemName) {
                            world.sendMessage("Please enter a valid item name.✨");
                            player.playSound('note.bass', { volume: 0.5 });
                            return;
                        }

                        const itemData = {
                            itemValue: response.formValues[1], // Quantity
                            rarity: response.formValues[2] // Rarity
                        };

                        MysticItemsDb.set(`${itemName}`, JSON.stringify(itemData));
                        world.sendMessage(`Item added: ${itemName} (Quantity: ${itemData.itemValue}, Rarity: ${itemData.rarity})✨`);
                        player.playSound('random.levelup', { volume: 0.8 });
                        return MysticAdmin_Main_Menu(player);

                    } catch (error) {
                        world.sendMessage(`Failed to add item: ${error.message}✨`);
                        player.playSound('note.bass', { volume: 0.5 });
                    }
                })();
                break;

            // ██████╗ ███████╗███╗   ███╗ ██████╗ ██╗   ██╗███████╗    ██╗████████╗███████╗███╗   ███╗███████╗    ███╗   ███╗███████╗███╗   ██╗██╗   ██╗
            // ██╔══██╗██╔════╝████╗ ████║██╔═══██╗██║   ██║██╔════╝    ██║╚══██╔══╝██╔════╝████╗ ████║██╔════╝    ████╗ ████║██╔════╝████╗  ██║██║   ██║
            // ██████╔╝█████╗  ██╔████╔██║██║   ██║██║   ██║█████╗      ██║   ██║   █████╗  ██╔████╔██║███████╗    ██╔████╔██║█████╗  ██╔██╗ ██║██║   ██║
            // ██╔══██╗██╔══╝  ██║╚██╔╝██║██║   ██║╚██╗ ██╔╝██╔══╝      ██║   ██║   ██╔══╝  ██║╚██╔╝██║╚════██║    ██║╚██╔╝██║██╔══╝  ██║╚██╗██║██║   ██║
            // ██║  ██║███████╗██║ ╚═╝ ██║╚██████╔╝ ╚████╔╝ ███████╗    ██║   ██║   ███████╗██║ ╚═╝ ██║███████║    ██║ ╚═╝ ██║███████╗██║ ╚████║╚██████╔╝
            // ╚═╝  ╚═╝╚══════╝╚═╝     ╚═╝ ╚═════╝   ╚═══╝  ╚══════╝    ╚═╝   ╚═╝   ╚══════╝╚═╝     ╚═╝╚══════╝    ╚═╝     ╚═╝╚══════╝╚═╝  ╚═══╝ ╚═════╝                                                                                                                                          
            case 2:
                (async () => {
                    const allMysticItemsDb = MysticItemsDb.getAll();
                    if (allMysticItemsDb.length === 0) {
                        world.sendMessage("No Items to edit or remove.✨");
                        player.playSound('note.bass', { volume: 0.5 });
                        return;
                    }
                    let page = 0;
                    const totalPages = Math.ceil(allMysticItemsDb.length / 5);
                    while (true) {
                        const start = page * 5;
                        const end = start + 5;

                        const form = new ActionFormData()
                            .title("Kittys Mystic Items Menu")
                            .body(`Select an Item to Edit or Remove:\nPage: ${page + 1} of ${totalPages}✨`);
                        allMysticItemsDb.slice(start, end).forEach((itemKey) => {
                            const itemData = JSON.parse(MysticItemsDb.get(itemKey));
                            const itemQuantity = itemData.itemValue;
                            const itemRarity = itemData.rarity;

                            let texturePath;
                            if (isitem.has(itemKey)) {
                                texturePath = `textures/items/${itemKey}`;
                            } else if (isblock.has(itemKey)) {
                                texturePath = `textures/blocks/${itemKey}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            form.button(` ${itemKey} (x${itemQuantity}) \n (Rarity: ${itemRarity})✨`, texturePath);
                        });
                        if (page > 0) {
                            form.button(`§o§1Last Page\n§3[ Page: ${page} ]✨`, "textures/ui/generic_dpad_left");
                        }
                        if (end < allMysticItemsDb.length) {
                            form.button(`§o§1Next Page\n§3[ Page: ${page + 2} ]✨`, "textures/ui/generic_dpad_right");
                        }
                        form.button("§l§cMain\nMenu✨", "textures/ui/icon_map");

                        const response = await form.show(player);
                        const selectedItemIndex = response.selection;
                        const totalButtonsOnPage = allMysticItemsDb.slice(start, end).length;
                        const hasPreviousButton = page > 0 ? 1 : 0;
                        const hasNextButton = end < allMysticItemsDb.length ? 1 : 0;

                        if (selectedItemIndex === totalButtonsOnPage + hasPreviousButton + hasNextButton) {
                            return MysticAdmin_Main_Menu(player);
                        }
                        if (page > 0 && selectedItemIndex === totalButtonsOnPage) {
                            page--;
                            continue;
                        }
                        if (end < allMysticItemsDb.length && selectedItemIndex === totalButtonsOnPage + hasPreviousButton) {
                            page++;
                            continue;
                        }

                        const itemToEdit = allMysticItemsDb[start + selectedItemIndex];
                        const itemData = JSON.parse(MysticItemsDb.get(itemToEdit));
                        const itemQuantity = itemData.itemValue;
                        const itemRarity = itemData.rarity;

                        try {
                            let texturePath;
                            if (isitem.has(itemToEdit)) {
                                texturePath = `textures/items/${itemToEdit}`;
                            } else if (isblock.has(itemToEdit)) {
                                texturePath = `textures/blocks/${itemToEdit}`;
                            } else {
                                texturePath = `textures/ui/gift_square`;
                            }

                            const EditForm = new ActionFormData()
                                .title('Mystic Edit Item Menu')
                                .body(`Edit the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`)
                                .button("Edit Quantity✨", "textures/ui/icon_armor")
                                .button("Edit Rarity✨", "textures/ui/icon_armor")
                                .button("Edit ID✨", "textures/ui/icon_armor")
                                .button(`Remove Item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r`, texturePath)
                                .button("Go Back\n[ Clear Item ]✨", "textures/ui/icon_map");

                            const editResponse = await EditForm.show(player);

                            if (editResponse.selection === 0) {
                                // Edit Item Quantity
                                const QuantityForm = new ModalFormData()
                                    .title("Edit Item Quantity")
                                    .slider(`Set new quantity for ${itemToEdit}✨`, 1, 64, 1, itemQuantity);
                                const quantityResponse = await QuantityForm.show(player);
                                const newQuantity = quantityResponse.formValues[0];

                                itemData.itemValue = newQuantity;
                                MysticItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Quantity updated for ${itemToEdit}: now x${newQuantity}✨`);

                            } else if (editResponse.selection === 1) {
                                // Edit Item Rarity
                                const RarityForm = new ModalFormData()
                                    .title("Edit Item Rarity")
                                    .slider(`0 = Rare\n100 = Common\nRarity✨`, 1, 100, 1, itemData.rarity);
                                const rarityResponse = await RarityForm.show(player);
                                const newRarity = rarityResponse.formValues[0];

                                itemData.rarity = newRarity;
                                MysticItemsDb.set(itemToEdit, JSON.stringify(itemData));
                                world.sendMessage(`Rarity updated for ${itemToEdit}: now ${newRarity}✨`);

                            } else if (editResponse.selection === 2) {
                                // Edit ID
                                const IdForm = new ModalFormData()
                                    .title("Edit Item ID")
                                    .textField(`Set new ID for ${itemToEdit}:`, "Enter new ID", `${itemToEdit}✨`);
                                const idResponse = await IdForm.show(player);
                                const newItemId = idResponse.formValues[0];

                                if (newItemId) {
                                    MysticItemsDb.del(itemToEdit); // Remove the old ID entry
                                    MysticItemsDb.set(newItemId, JSON.stringify(itemData)); // Save with new ID
                                    world.sendMessage(`Item ID updated: ${itemToEdit} → ${newItemId}✨`);
                                    return
                                } else {
                                    world.sendMessage("Invalid item ID.✨");
                                }
                            } else if (editResponse.selection === 3) {
                                // Confirm Removal
                                const ConfirmForm = new ActionFormData()
                                    .title('Mystic Confirm Item Removal')
                                    .body(`Are you sure you want to remove the item: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r?✨`)
                                    .button(`Remove: §l${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})§r✨`, texturePath)
                                    .button("Go Back✨", "textures/ui/icon_map");

                                const confirmResponse = await ConfirmForm.show(player);

                                if (confirmResponse.selection === 0) {
                                    MysticItemsDb.del(itemToEdit);
                                    world.sendMessage(`Item removed: ${itemToEdit} (x${itemQuantity}) (Rarity: ${itemRarity})✨`);
                                    player.playSound('note.bass', { volume: 0.5 });
                                }
                            }
                        } catch (error) {
                            world.sendMessage("Error occurred while showing the edit menu.✨", error);
                            player.playSound('note.bass', { volume: 0.5 });
                        }
                    }
                })();
                break;
        }
    });
}
async function MysticcloseCrateMenu(player) {
    await player.sendMessage('§6§lCrate menu closed. Thank you!✨');
    await player.playSound('note.bass', { volume: 0.5 });
}

// ██╗    ██╗ ██████╗ ██████╗ ██╗     ██████╗     ███████╗██╗   ██╗███████╗███╗   ██╗████████╗███████╗
// ██║    ██║██╔═══██╗██╔══██╗██║     ██╔══██╗    ██╔════╝██║   ██║██╔════╝████╗  ██║╚══██╔══╝██╔════╝
// ██║ █╗ ██║██║   ██║██████╔╝██║     ██║  ██║    █████╗  ██║   ██║█████╗  ██╔██╗ ██║   ██║   ███████╗
// ██║███╗██║██║   ██║██╔══██╗██║     ██║  ██║    ██╔══╝  ╚██╗ ██╔╝██╔══╝  ██║╚██╗██║   ██║   ╚════██║
// ╚███╔███╔╝╚██████╔╝██║  ██║███████╗██████╔╝    ███████╗ ╚████╔╝ ███████╗██║ ╚████║   ██║   ███████║
//  ╚══╝╚══╝  ╚═════╝ ╚═╝  ╚═╝╚══════╝╚═════╝     ╚══════╝  ╚═══╝  ╚══════╝╚═╝  ╚═══╝   ╚═╝   ╚══════╝
world.beforeEvents.playerInteractWithEntity.subscribe(async (data) => {
    const { player, target } = data
    if (target.typeId != "kittys:mysticchest") return
    if (player && data.itemStack && data.itemStack.typeId === "kittys:removal_tool") {
        return;
    };
    const MysticcrateTimerValue = MysticSettingsDb.get('KittysMysticCrateSettings:MysticCrateTimer') || '2';
    const MysticcrateTimerHours = Number(MysticcrateTimerValue);
    const cooldownTime = MysticcrateTimerHours * 60 * 60 * 1000; // hours to milliseconds
    data.cancel = true;
    system.run(() => {
        target.dimension.playSound("random.pop", target.location, { pitch: 0.5 })
    })
    if (
        data.player.hasTag("Admin") ||
        data.player.hasTag("admin") ||
        data.player.hasTag("Staff") ||
        data.player.hasTag("staff") ||
        data.player.hasTag("Owner") ||
        data.player.hasTag("owner")
    ) {
        await MysticAdmin_Main_Menu(data.player);
        return;
    }
    if (player && data.itemStack && data.itemStack.typeId === "kittys:mysticcrate_key") {
        await data.player.sendMessage('§6§lOpening Crate Menu!✨');
        system.run(() => {
            target.dimension.runCommand(`clear "${player.name}" kittys:mysticcrate_key 0 1`)
            target.dimension.runCommand(`say §c§lMystic Key Used!✨`);

            target.dimension.runCommand(`tag "${player.name}" add Mystickeyed`)
        });

        const MysticcrateMenu = new ActionFormData()
            .title('!Mystic Crate')
            .body('Your Daily Crate Awaits!')
            .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
            .button('Close', 'textures/ui/cancel');

        try {
            const result = await forceShow(data.player, MysticcrateMenu);
            result.selection === 0 ? await MysticCrate_Form(data.player) : await MysticcloseCrateMenu(data.player);
        } catch (error) {
            console.error(`Error displaying crate main menu: ${error.message}`);
        }
        return
    }
    function isOnCooldown(player) {
        const now = Date.now();
        if (!Mysticcooldowns.has(player.name)) return false;
        const cooldown = Mysticcooldowns.get(player.name);
        return (now - cooldown) < cooldownTime;
    }
    const MysticsavedSettings = getMysticSettingsDb();
    const ToggleMysticCoolDownSystemValue = MysticsavedSettings.ToggleMysticCoolDownSystemcommand === 'true';
    if (ToggleMysticCoolDownSystemValue === true) {
        await data.player.sendMessage(`§c§lYou Need A Crate Key!✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }
    if (isOnCooldown(data.player)) {
        const remainingTime = cooldownTime - (Date.now() - Mysticcooldowns.get(data.player.name));
        const remainingSeconds = Math.ceil(remainingTime / 1000);
        const minutes = Math.floor(remainingSeconds / 60);
        const seconds = remainingSeconds % 60;
        const remainingTimeMessage = `${minutes} minute${minutes !== 1 ? 's' : ''} ${seconds} second${seconds !== 1 ? 's' : ''}`;
        await data.player.sendMessage(`§c§lPlease wait ${remainingTimeMessage} before opening the mystic crate again.✨`);
        data.player.playSound('note.bass', { volume: 0.5 });
        return;
    }

    await data.player.sendMessage('§6§lOpening Crate Menu!✨');

    const MysticcrateMenu = new ActionFormData()
        .title('!Mystic Crate')
        .body('Your Daily Crate Awaits!')
        .button('Collect Prize', 'textures/ui/accessibility_glyph_color')
        .button('Close', 'textures/ui/cancel');

    try {
        const result = await forceShow(data.player, MysticcrateMenu);
        result.selection === 0 ? await MysticCrate_Form(data.player) : await MysticcloseCrateMenu(data.player);
    } catch (error) {
        console.error(`Error displaying crate main menu: ${error.message}`);
    }
});

// 888    d8P  d8b 888    888                   .d8888b.  888      d8b                        888888b.            d8b 888      888               888
// 888   d8P   Y8P 888    888                  d88P  Y88b 888      Y8P                        888  "88b           Y8P 888      888               888
// 888  d8P        888    888                  Y88b.      888                                 888  .88P               888      888               888
// 888d88K     888 888888 888888 888  888       "Y888b.   88888b.  888 88888888 88888888      8888888K.  888  888 888 888  .d88888 .d8888b       888
// 8888888b    888 888    888    888  888          "Y88b. 888 "88b 888    d88P     d88P       888  "Y88b 888  888 888 888 d88" 888 88K           888
// 888  Y88b   888 888    888    888  888            "888 888  888 888   d88P     d88P        888    888 888  888 888 888 888  888 "Y8888b.      Y8P
// 888   Y88b  888 Y88b.  Y88b.  Y88b 888      Y88b  d88P 888  888 888  d88P     d88P         888   d88P Y88b 888 888 888 Y88b 888      X88       "
// 888    Y88b 888  "Y888  "Y888  "Y88888       "Y8888P"  888  888 888 88888888 88888888      8888888P"   "Y88888 888 888  "Y88888  88888P'      888
//                                    888
//                               Y8b d88P
//                                "Y88P"                                                                                                             